# Author: Salyl Bhagwat, Gammath Works
# Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works
# All Rights Reserved
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__author__ = 'Salyl Bhagwat'
__copyright__ = 'Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works'

import pandas as pd
import numpy as np


def get_price_signals(tsymbol, df, df_summ):

    MIN_TRADING_DAYS_PER_YEAR = 249
    PRICE_PERCENT_CUTOFF = 85

    price_gscore = 0
    price_max_score = 0
    price_signals = ''
    curr_count_quantile_str = ''
    curr_price_1y_quantile_str = ''

    prices = df.Close
    prices_len = len(prices)
    if (prices_len <= 0):
        raise ValueError('Invalid Price data for generating signals')

    lp = prices[prices_len-1]

    #Limit to 3 digits after decimal point
    curr_price = f'curr_price: %5.3f' % lp

    nftwlh = ''

    #Get consecutive falling and rising days count
    last_falling_days_count = 0
    max_falling_days_count = 0
    last_rising_days_count = 0
    max_rising_days_count = 0

    price_consec_falling_days_count_series = pd.Series(np.nan, pd.RangeIndex(prices_len))
    price_consec_falling_days_count_index = 0

    price_consec_rising_days_count_series = pd.Series(np.nan, pd.RangeIndex(prices_len))
    price_consec_rising_days_count_index = 0

    for i in range(prices_len-1):
        if (prices[i] > prices[i+1]):
            last_falling_days_count += 1

            if (last_rising_days_count > 0):
                price_consec_rising_days_count_index += 1

            last_rising_days_count = 0

            price_consec_falling_days_count_series[price_consec_falling_days_count_index] = last_falling_days_count

            if (last_falling_days_count > max_falling_days_count):
                max_falling_days_count = last_falling_days_count
        elif (prices[i] < prices[i+1]):
            last_rising_days_count += 1

            if (last_falling_days_count > 0):
                price_consec_falling_days_count_index += 1

            last_falling_days_count = 0

            price_consec_rising_days_count_series[price_consec_rising_days_count_index] = last_rising_days_count

            if (last_rising_days_count > max_rising_days_count):
                max_rising_days_count = last_rising_days_count
        else:
            last_falling_days_count = 0
            last_rising_days_count = 0

    price_consec_falling_days_count_series = price_consec_falling_days_count_series.dropna()

    #Lower counts will be too many so drop them to get approx percentile past lower numbers
    price_consec_falling_days_count_series = price_consec_falling_days_count_series.drop_duplicates()
    price_consec_falling_days_count_series = price_consec_falling_days_count_series.sort_values()

    price_consec_rising_days_count_series = price_consec_rising_days_count_series.dropna()

    #Lower counts will be too many so drop them to get approx percentile past lower numbers
    price_consec_rising_days_count_series = price_consec_rising_days_count_series.drop_duplicates()
    price_consec_rising_days_count_series = price_consec_rising_days_count_series.sort_values()

    try:
        #50-day average
        fiftyDayAverage = df_summ['fiftyDayAverage'][0]
    except:
        fiftyDayAverage = 0

    try:
        #200-day average
        twoHundredDayAverage = df_summ['twoHundredDayAverage'][0]
    except:
        twoHundredDayAverage = 0

    try:
        #52-week low
        yearly_lowest_val = df_summ['fiftyTwoWeekLow'][0]
    except:
        yearly_lowest_val = 0

    try:
        #52-week high
        yearly_highest_val = df_summ['fiftyTwoWeekHigh'][0]
    except:
        yearly_highest_val = 0

    if (yearly_lowest_val > 0):
        if (lp <= yearly_lowest_val):
            #New 52-week low; Log it for information
            nftwlh = 'new fiftyTwoWeekLow'

    if (yearly_highest_val <= 0):
        #New 52-week high; Log it for information
        nftwlh = 'new fiftyTwoWeekHigh'

    if (last_falling_days_count > 0):
        price_dir = 'falling'
        #Get percentile values
        bp, mp, tp = price_consec_falling_days_count_series.quantile([0.25, 0.5, 0.75])

        #Log quantile for days count
        if (last_falling_days_count < mp):
            curr_count_quantile_str = 'day-count in bottom quantile'
        elif (last_falling_days_count < tp):
            curr_count_quantile_str = 'day-count in middle quantile'
        else:
            curr_count_quantile_str = 'day-count in top quantile'

        price_gscore -= 1

        if (last_falling_days_count >= bp):
            price_gscore -= 2

        #Checkout price wrt 50-day average, 200-day average and 52-week low
        if (fiftyDayAverage > 0):
            if (lp > fiftyDayAverage):
                price_gscore -= 1

        if (twoHundredDayAverage > 0):
            if (lp > twoHundredDayAverage):
                price_gscore -= 1

        if (yearly_highest_val > 0):
            pct_val = lp*100/yearly_highest_val

            #If we are closer to 52-week high then indicate premium; else indicate the discount score
            if (pct_val >= PRICE_PERCENT_CUTOFF):
                price_gscore -= 3

    elif (last_rising_days_count > 0):
        price_dir = 'rising'
        bp, mp, tp = price_consec_rising_days_count_series.quantile([0.25, 0.5, 0.75])

        #Log quantile for days count
        if (last_rising_days_count < mp):
            curr_count_quantile_str = 'day-count in bottom quantile'
        elif (last_rising_days_count < tp):
            curr_count_quantile_str = 'day-count in middle quantile'
        else:
            curr_count_quantile_str = 'day-count in top quantile'

        price_gscore += 1
        if (last_rising_days_count <= bp):
            price_gscore += 2

        #Checkout price wrt 50-day average, 200-day average and 52-week low
        if (fiftyDayAverage > 0):
            if (lp <= fiftyDayAverage):
                price_gscore += 1

        if (twoHundredDayAverage > 0):
            if (lp <= twoHundredDayAverage):
                price_gscore += 1

        if (yearly_lowest_val > 0):
            pct_val = yearly_lowest_val*100/lp

            #If current price is not too far from 52-week low then increase discount score
            if (pct_val >= PRICE_PERCENT_CUTOFF):
                price_gscore += 3
    else:
        price_dir = 'unclear'

    price_max_score += 8

    #Get current price percentiles for most recent 52-week range
    one_year_prices = df['Close'][(prices_len-MIN_TRADING_DAYS_PER_YEAR):]
    bp, mp, tp = one_year_prices.quantile([0.25, 0.5, 0.75])

    #Log quantile for current price in 52-week range
    if (lp < mp):
        curr_price_1y_quantile_str = 'in 1Y bottom quantile'
    elif (lp < tp):
        curr_price_1y_quantile_str = 'in 1Y middle quantile'
    else:
        curr_price_1y_quantile_str = 'in 1Y top quantile'

    #If price is in lower percentile then higher buy score
    #If price is in higher percentile then higher sell score
    if ((lp <= bp) or (lp >= tp)):
        if (lp <= bp):
            price_gscore += 2
        elif (lp >= tp):
            price_gscore -= 2
    else:
        if ((lp > bp) and (lp < mp)):
            price_gscore += 1

    price_max_score += 2

    price_grec = f'price_gscore:{price_gscore}/{price_max_score}'

    price_signals = f'price: {price_dir},{curr_count_quantile_str},{curr_price}:{curr_price_1y_quantile_str},{price_grec},{nftwlh}'

    return price_gscore, price_max_score, price_signals



# Author: Salyl Bhagwat, Gammath Works
# Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works
# All Rights Reserved
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__author__ = 'Salyl Bhagwat'
__copyright__ = 'Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works'

import pandas as pd
import numpy as np

def get_pbr_signals(tsymbol, df_summ):

    pbr_gscore = 0
    pbr_max_score = 0

    try:
        if (len(df_summ) > 0):
            try:
                pbr = df_summ['priceToBook'][0]
                if (np.isnan(pbr)):
                    PBR_string = 'No PBR data'
                else:
                    pbr = round(pbr, 3)
                    PBR_string = f'{pbr}'
                    if (pbr > 0):

                        #Lower PBR is better; Not giving more weight as we have analyst reco and other factors accouting for "selection" criteria
                        if (pbr < 20):
                            pbr_gscore += 1
                        else:
                            pbr_gscore -= 1
            except:
                raise ValueError('PBR not found')

        else:
            raise RuntimeError('Dataframe empty')

    except:
        pbr = 0
        PBR_string = 'No PBR data'

    pbr_max_score += 1

    pbr_grec = f'pbr_gscore:{pbr_gscore}/{pbr_max_score}'

    pbr_signals = f'PBR:{PBR_string},{pbr_grec}'

    return pbr_gscore, pbr_max_score, pbr_signals



# Author: Salyl Bhagwat, Gammath Works
# Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works
# All Rights Reserved
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__author__ = 'Salyl Bhagwat'
__copyright__ = 'Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works'

from pathlib import Path
import pandas as pd

#Get percentage held by institutions for tsymbol
def get_ihp_signals(tsymbol, df_summ):

    ihp_gscore = 0
    ihp_max_score = 0

    try:
        #Get data about percentage held from summary dataframe
        ihp = df_summ['heldPercentInstitutions'][0]
    except:
        raise ValueError('heldPercentInstitutions value not found')

    #We can do checks for different levels but for now this will suffice
    if (ihp > 0):
        if (ihp > 0.7):
            ihp_gscore += 1
        else:
            ihp_gscore -= 1

    ihp_max_score += 1

    #Round it off to take less space when displaying
    ihp = round(ihp, 3)

    #At some point, we can add percent change. Right now requires to be checked using local old val with new val; REVISIT

    ihp_grec = f'ihp_gscore:{ihp_gscore}/{ihp_max_score}'

    ihp_signals = f'IHP:{ihp},{ihp_grec}'

    return ihp_gscore, ihp_max_score, ihp_signals



# Author: Salyl Bhagwat, Gammath Works
# Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works
# All Rights Reserved
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__author__ = 'Salyl Bhagwat'
__copyright__ = 'Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works'

import yfinance as yf
import pandas as pd
from datetime import datetime
from pathlib import Path
from talib import RSI, BBANDS, MACD, MFI, STOCH
from matplotlib import pyplot as plt
import sys
import os
import time
try:
    from gammath_spot import gammath_utils as gut
    from gammath_spot import gammath_kf_signals as gkf
    from gammath_spot import gammath_ols_signals as gols
except:
    import gammath_utils as gut
    import gammath_kf_signals as gkf
    import gammath_ols_signals as gols

def plot_and_save_charts(tsymbol, bb_df, rsi_df, mfi_df, macd_df, stoch_df, kf_df, ols_df):

    RSI_OVERSOLD_LEVEL = 30
    RSI_OVERBOUGHT_LEVEL = 70

    MFI_OVERSOLD_LEVEL = 20
    MFI_OVERBOUGHT_LEVEL = 80

    STOCH_OVERSOLD_LEVEL = 20
    STOCH_OVERBOUGHT_LEVEL = 80

    Tickers_dir = Path('tickers')

    path = Tickers_dir / f'{tsymbol}'

    file_exists = (path / f'{tsymbol}_charts.png').exists()

    #Check if file exists and is it from another day
    if file_exists:
        fstat = os.stat(path / f'{tsymbol}_charts.png')

        if (True == gut.check_if_same_day(fstat)):
            return

    #Draw the charts to view all at once as subplots
    figure, axes = plt.subplots(nrows=7, figsize=(21, 19))

    try:
        bb_df.plot(ax=axes[0],lw=1,title='Bollinger Bands')
        rsi_df.plot(ax=axes[1],lw=1,title='Relative Strength Index')
        axes[1].axhline(RSI_OVERBOUGHT_LEVEL,lw=1,ls='-',c='r')
        axes[1].axhline(RSI_OVERSOLD_LEVEL,lw=1,ls='-',c='r')
        macd_df.plot(ax=axes[2],lw=1,title='Moving Average Convergence Divergence')
        mfi_df.plot(ax=axes[3],lw=1,title='Money Flow Index')
        axes[3].axhline(MFI_OVERBOUGHT_LEVEL,lw=1,ls='-',c='r')
        axes[3].axhline(MFI_OVERSOLD_LEVEL,lw=1,ls='-',c='r')
        stoch_df.plot(ax=axes[4],lw=1,title='Stochastic Slow')
        axes[4].axhline(STOCH_OVERBOUGHT_LEVEL,lw=1,ls='-',c='r')
        axes[4].axhline(STOCH_OVERSOLD_LEVEL,lw=1,ls='-',c='r')
        kf_df.plot(ax=axes[5], lw=1,title='Kalman Filter')
        ols_df.plot(ax=axes[6], lw=1,title='OLS')
        plt.savefig(path / f'{tsymbol}_charts.png')
    except:
        print('\nERROR: Plotting charts for ', tsymbol, ': ', sys.exc_info()[0])




# Author: Salyl Bhagwat, Gammath Works
# Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works
# All Rights Reserved
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__author__ = 'Salyl Bhagwat'
__copyright__ = 'Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works'

import pandas as pd
import yfinance as yf
from datetime import datetime
from pathlib import Path
import sys


def get_ticker_history(tsymbol, ticker, path):

    if (len(tsymbol) == 0):
        raise ValueError('Invalid symbol')

    #Get summary of past 5 years history
    end_date = datetime.today()
    start_date = datetime(end_date.year-5, end_date.month, end_date.day)

    try:
        # Get 5Y stock history based on dates
#        stock_history = ticker.history(interval='1d', start=start_date, end=end_date, actions=True,auto_adjust=True)

        # Get 5Y stock history using 5Y-period
        stock_history = ticker.history(period='5y', interval='1d', actions=True,auto_adjust=True)
    except:
        raise RuntimeError('Error obtaining stock history')

    stock_history_len = len(stock_history)

    if (stock_history_len > 0):
        if not path.exists():
            path.mkdir(parents=True, exist_ok=True)

        try:
            update_file = True

            #Always save the original price history for reference
            stock_history.to_csv(path / f'{tsymbol}_history_orig.csv')

            #Entries with nan values cause problems during analysis so drop those
            #Save the new history after dropping rows with nan values
            stock_history.dropna(how='any').to_csv(path / f'{tsymbol}_history_orig_changed.csv')

            #Read old history for comparison (if it exists)
            history_file_exists = (path / f'{tsymbol}_history.csv').exists()

            if (history_file_exists):
                #Get the len of existng file without nan-rows
                df_old = pd.read_csv(path / f'{tsymbol}_history.csv')
                df_old_len = len(df_old)

                #Get the len of new file without nan-rows
                df_new_changed = pd.read_csv(path / f'{tsymbol}_history_orig_changed.csv')
                df_new_changed_len = len(df_new_changed)

                if (df_new_changed_len < df_old_len):
                    print(f'\nNew file is shorter. New len: {df_new_len}, Old len: {df_old_len}')
                    # This is a workaround for cases where history file has missing data for
                    # few days prior to last day. In such cases, last day's data exists.
                    # This could cause incorrect analysis so I'm maintaining older file and
                    # using that where only last day's data doesn't exist.
                    # A Note in the CSV with overall gScores will indicate that today's
                    # data is missing so result can be ignored or today's data can be
                    # rechecked manually or in such cases, re-running scraper seems to
                    # get proper history and then this workaroud won't be used

                    # Don't replace old file as it has more data.
                    update_file = False

            if (update_file):
                #Save the history file without nan-rows for processing
                stock_history.dropna(how='any').to_csv(path / f'{tsymbol}_history.csv')

        except:
            raise RuntimeError('Stock history file RW error')
    else:
        raise ValueError('Invalid length stock history')

    return



# Author: Salyl Bhagwat, Gammath Works
# Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works
# All Rights Reserved
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__author__ = 'Salyl Bhagwat'
__copyright__ = 'Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works'

import pandas as pd
from pykalman import KalmanFilter
import numpy as np

def get_kf_state_means(tsymbol, df):

    kf_gscore = 0
    kf_max_score = 0
    kf_signals = ''

    curr_count_quantile_str = ''
    curr_diff_quantile_str = ''

    prices = df.Close
    prices_len = len(prices)
    sm_len = 0

    #Generate state means based on closing prices
    if (prices_len > 0):
        try:
            #Initialize Kalman filter with default params
            kf = KalmanFilter()

            #Get the state mean and covariance
            state_means, state_covariance = kf.filter(df.Close)
        except:
            raise RuntimeError('Failed to generate Kalman Filter State Means and Covariance')

        #Extract state means into PD series
        ds_sm = pd.Series(state_means.flatten())
        sm_len = len(ds_sm)

    if ((sm_len <= 0) or (prices_len <= 0)):
        raise ValueError('Incorrect length of data frame')

    #Get the most recent state mean to compare with most recent price
    last_sm = ds_sm[sm_len-1]
    last_price = prices[prices_len-1]

    #Use historical levels' comparison to current levels for computing buy/sell scores
    curr_below_mean_count = 0
    curr_above_mean_count = 0

    #init the below mean count PD series
    kf_below_mean_count_series = pd.Series(np.nan, pd.RangeIndex(sm_len))

    #init the above mean count PD series
    kf_above_mean_count_series = pd.Series(np.nan, pd.RangeIndex(sm_len))

    kf_below_mean_count_index = 0
    kf_above_mean_count_index = 0

    kf_neg_diff_index = 0
    kf_pos_diff_index = 0
    curr_diff = 0

    #Init the -ve diff PD series
    kf_neg_diff_series = pd.Series(np.nan, pd.RangeIndex(sm_len))

    #Init the +ve diff PD series
    kf_pos_diff_series = pd.Series(np.nan, pd.RangeIndex(sm_len))

    #Collect historical below/above mean days and diff stats (ignoring first 20 entries as means aren't at the desired values)
    for i in range(20, sm_len):

        price = prices[i]
        sm = ds_sm[i]

        if (price <= sm):
            #Price is below state mean

            #Compute current price diff compared to state mean
            curr_diff = (sm - price)

            #Save for statistical analysis
            kf_neg_diff_series[kf_neg_diff_index] = curr_diff
            kf_neg_diff_index += 1
            curr_below_mean_count += 1

            #Check if previous iteration was above mean and if so update and save counts
            if (curr_above_mean_count > 0):
                kf_above_mean_count_series[kf_above_mean_count_index] = curr_above_mean_count
                kf_above_mean_count_index += 1
                curr_above_mean_count = 0
        else:
            #Price is above state mean

            #Compute current price diff compared to state mean
            curr_diff = (price - sm)

            #Save for statistical analysis
            kf_pos_diff_series[kf_pos_diff_index] = curr_diff
            kf_pos_diff_index += 1
            curr_above_mean_count += 1

            #Check if previous iteration was below mean and if so update and save counts
            if (curr_below_mean_count > 0):
                kf_below_mean_count_series[kf_below_mean_count_index] = curr_below_mean_count
                kf_below_mean_count_index += 1
                curr_below_mean_count = 0

    kf_neg_diff_series = kf_neg_diff_series.dropna()
    kf_neg_diff_series = kf_neg_diff_series.sort_values()

    kf_pos_diff_series = kf_pos_diff_series.dropna()
    kf_pos_diff_series = kf_pos_diff_series.sort_values()

    #Use current diff to factor into dip score based on which quantile it falls in
    if (curr_below_mean_count > 0):

        #Get values at 25/50/75 percentile for -ve diffs
        bp, mp, tp = kf_neg_diff_series.quantile([0.25, 0.5, 0.75])

        #bigger the difference could mean better price compared to mean
        #scores are based on curr_diff greater than 25, 50, 75 percentile
        if (curr_diff < mp):
            curr_diff_quantile_str = 'bottom quantile'

        if (curr_diff >= bp):
            kf_gscore += 2

        kf_max_score += 2

        if (curr_diff >= mp):
            kf_gscore += 2
            curr_diff_quantile_str = 'middle quantile'

        kf_max_score += 2

        if (curr_diff >= tp):
            kf_gscore += 3
            curr_diff_quantile_str = 'top quantile'

        kf_max_score += 3

    else:

        #Get values at 25/50/75 percentile for +ve diffs
        bp, mp, tp = kf_pos_diff_series.quantile([0.25, 0.5, 0.75])

        #bigger the difference could mean better price compared to mean
        #scores are based on curr_diff greater than 25, 50, 75 percentile
        if (curr_diff < mp):
            curr_diff_quantile_str = 'bottom quantile'

        if (curr_diff >= bp):
            kf_gscore -= 2

        kf_max_score += 2

        if (curr_diff >= mp):
            kf_gscore -= 2
            curr_diff_quantile_str = 'middle quantile'

        kf_max_score += 2

        if (curr_diff >= tp):
            kf_gscore -= 3
            curr_diff_quantile_str = 'top quantile'

        kf_max_score += 3

    kf_below_mean_count_series = kf_below_mean_count_series.dropna()
    kf_below_mean_count_series = kf_below_mean_count_series.sort_values()

    kf_above_mean_count_series = kf_above_mean_count_series.dropna()
    kf_above_mean_count_series = kf_above_mean_count_series.sort_values()

    #Compute buy/sell scores based on where current below mean count falls in 25, 50, 75 percentile
    if (curr_below_mean_count > 0):
        #Get percentile values for below mean counts
        bp, mp, tp = kf_below_mean_count_series.quantile([0.25, 0.5, 0.75])

        if (curr_below_mean_count < mp):
            curr_count_quantile_str = 'bottom quantile'

        if (curr_below_mean_count >= bp):
            kf_gscore += 1

        kf_max_score += 1

        if (curr_below_mean_count >= mp):
            kf_gscore += 1
            curr_count_quantile_str = 'middle quantile'

        kf_max_score += 1

        if (curr_below_mean_count >= tp):
            kf_gscore += 1
            curr_count_quantile_str = 'top quantile'

        kf_max_score += 1

    else:
        #Get percentile values for above mean counts
        bp, mp, tp = kf_above_mean_count_series.quantile([0.25, 0.5, 0.75])

        if (curr_above_mean_count < mp):
            curr_count_quantile_str = 'bottom quantile'

        if (curr_above_mean_count >= bp):
            kf_gscore -= 1

        kf_max_score += 1

        if (curr_above_mean_count >= mp):
            kf_gscore -= 1
            curr_count_quantile_str = 'middle quantile'

        kf_max_score += 1

        if (curr_above_mean_count >= tp):
            kf_gscore -= 1
            curr_count_quantile_str = 'top quantile'

        kf_max_score += 1

    kf_grec = f'kf_gscore:{kf_gscore}/{kf_max_score}'

    if (curr_below_mean_count > 0):
        kf_signals = f'KF:-ve_days:{curr_count_quantile_str},-ve_diff:{curr_diff_quantile_str},{kf_grec}'
    else:
        kf_signals = f'KF:+ve_days:{curr_count_quantile_str},+ve_diff:{curr_diff_quantile_str},{kf_grec}'


    #Return KF state means in a dataframe for plotting charts
    kf_df = pd.DataFrame({tsymbol: df.Close, 'Kalman Filter': ds_sm})

    return kf_df, kf_gscore, kf_max_score, kf_signals



# Author: Salyl Bhagwat, Gammath Works
# Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works
# All Rights Reserved
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__author__ = 'Salyl Bhagwat'
__copyright__ = 'Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works'

import multiprocessing as mp
from multiprocessing import Process
try:
    from gammath_spot import gammath_get_stocks_data as ggsd
    from gammath_spot import gammath_utils as gut
except:
    import gammath_get_stocks_data as ggsd
    import gammath_utils as gut

import sys
from pathlib import Path
import pandas as pd

def main():
    """
    Main function to scrape the web and collect data necessary for analyzing and computing gScores for each stock in the provided watchlist. It saves the collected data in tickers/<ticker_symbol> director.
    """

    #Avoiding to check number of args as if watchlist is not there then there will be an exception anyway
    try:
        #Get the watchlist file from pgm argument
        sf_name = sys.argv[1]
    except:
        print('ERROR: Need watch list file name as one argument to this Program. See watchlist.csv')
        raise ValueError('Missing watch list')

    #Read the watchlist
    try:
        watch_list = pd.read_csv(sf_name)
    except:
        print('ERROR: Failed to read watchlist. See watchlist.csv for example')
        raise ValueError('Watchlist file read failed')

    #Set the start method for launching parallel processes
    #Python 3.8 onwards 'spawn' is the default method for MacOS and is supported on Linux and Windows
    #so using it for portability. Spawn method is much slower compared to 'fork' method. If there are no unsafe changes made to this project then on MacOS and Linux this can be changed to use 'fork'
    mp.set_start_method('spawn')

    #Check how many cores we have to be able to run parallel
    core_count = mp.cpu_count()

    #Need to check portability on this
    #Might need to reduce the number cores actually used hence core_count and cores_to_use are defined separately
    cores_to_use = core_count

    if (cores_to_use < 1):
        cores_to_use = 1

    proc_handles = []

    max_tickers = len(watch_list)

    #Instantiate GUTILS class
    gutils = gut.GUTILS()

    #Fetch and save S&P500 list.
    gutils.get_sp500_list()

    #Instances of GSD class
    gsd_instances = []

    #One process per ticker symbol
    #Run cores_to_use number of processes in parallel
    start_index = 0
    if (max_tickers > cores_to_use):
        end_index = cores_to_use
    else:
        end_index = max_tickers

    while (max_tickers):
        for i in range(start_index, end_index):
            sym = watch_list['Symbol'][i].strip()
            gsd_instances.append(ggsd.GSD())
            proc_handles.append(Process(target=gsd_instances[i].get_stocks_data, args=(f'{sym}',)))
            proc_handles[i].start()

            max_tickers -= 1

        for i in range(start_index, end_index):
            proc_handles[i].join()

        #Running out of resources so need to close handles and release resources
        for i in range(start_index, end_index):
            proc_handles[i].close()

        if (max_tickers):
            start_index = end_index
            if (max_tickers > cores_to_use):
                end_index += cores_to_use
            else:
                end_index += max_tickers

    #Aggregate and save PE data
    gutils.aggregate_pe_data()

if __name__ == '__main__':
    main()



# Author: Salyl Bhagwat, Gammath Works
# Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works
# All Rights Reserved
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__author__ = 'Salyl Bhagwat'
__copyright__ = 'Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works'

import yfinance as yf
from pathlib import Path
import bisect
from datetime import datetime
import pandas as pd
import time
import os
import numpy as np

def get_options_signals(tsymbol, path, curr_price, df_summ):

    options_gscore = 0
    options_max_score = 0
    shortRatio = 0

    #Get the options data from existing file for next options expiry
    if ((path / f'{tsymbol}_options_dates.csv').exists()):

        #Get the latest options expiry date
        dates_df = pd.read_csv(path / f'{tsymbol}_options_dates.csv', index_col='Unnamed: 0')

        #Next expiry date
        option_date = dates_df.loc[0][0]

        #Get the calls and puts info
        df_calls = pd.read_csv(path / f'{tsymbol}_call_{option_date}.csv')
        df_puts = pd.read_csv(path / f'{tsymbol}_put_{option_date}.csv')

        try:
            call_opts_size = len(df_calls)
            put_opts_size = len(df_puts)

            if ((call_opts_size != 0) and (put_opts_size != 0)):

                #Get index for current price in call options data
                i = bisect.bisect_right(df_calls['strike'], curr_price)

                if (i < (call_opts_size-1)):
                    bullish_start_index = i
                else:
                    bullish_start_index = 0

                #Get index for current price in put options data
                i = bisect.bisect_right(df_puts['strike'], curr_price)

                if (i < (put_opts_size-1)):
                    bearish_end_index = i
                else:
                    bearish_end_index = 0

                #Get total call open interest for curr price and higher
                total_bullish_open_interest = df_calls['openInterest'][bullish_start_index:call_opts_size-1].sum()

                #Get total put open interest for curr price and lower
                total_bearish_open_intetest = df_puts['openInterest'][0:bearish_end_index].sum()

                if (total_bullish_open_interest > total_bearish_open_intetest):
                    options_gscore += 4
                    calls_puts_string = 'Bullish'
                else:
                    options_gscore -= 4
                    calls_puts_string = 'Bearish'
            else:
                calls_puts_string = 'No calls, puts data'

        except:
            calls_puts_string = 'No calls, puts data'
    else:
        calls_puts_string = 'No calls, puts data'

    #Just immediate options expiry data so not putting too much weight
    options_max_score += 4

    #Use short ratio to read the bullish/bearish trend
    #Add weightage for buy/sell scores
    try:
        shortRatio = df_summ['shortRatio'][0]
        if (np.isnan(shortRatio)):
            short_ratio_string = 'No short_ratio data'
        else:
            short_ratio_string = f'short_ratio:{shortRatio}'
            if (shortRatio > 0):
                if (shortRatio < 3):
                    options_gscore += 6
                elif (shortRatio < 6):
                    options_gscore += 4
                elif (shortRatio < 10):
                    options_gscore += 2
                else:
                    options_gscore -= 6
            else:
                options_gscore -= 6

    except:
        short_ratio_string = 'No short_ratio data'

    options_max_score += 6

    options_grec = f'options_gscore:{options_gscore}/{options_max_score}'
    options_signals = f'options: {short_ratio_string},{calls_puts_string},{options_grec}'
    
    return options_gscore, options_max_score, options_signals




# Author: Salyl Bhagwat, Gammath Works
# Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works
# All Rights Reserved
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__author__ = 'Salyl Bhagwat'
__copyright__ = 'Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works'

import pandas as pd
from pathlib import Path
from talib import RSI
import numpy as np


def get_rsi_signals(tsymbol, df, path):

    RSI_TIME_PERIOD = 14
    RSI_OVERSOLD_LEVEL = 30
    RSI_OVERBOUGHT_LEVEL = 70

    rsi_gscore = 0
    rsi_max_score = 0
    rsi_signals = ''
    curr_count_quantile_str = ''

    try:
        rsi = RSI(df.Close, timeperiod=RSI_TIME_PERIOD)
    except:
        raise RuntimeError('RSI call failed')

    rsi_len = len(rsi)
    if (rsi_len <= 0):
        raise ValueError('Invalid length of RSI data')

    rsi_ds = rsi.describe()
    curr_rsi = rsi[rsi_len-1]
    prev_rsi = rsi[rsi_len-2]

    if (curr_rsi < prev_rsi):
        rsi_direction = 'falling'
    elif (curr_rsi > prev_rsi):
        rsi_direction = 'rising'
    else:
        rsi_direction = 'direction_unclear'

    #Get the RSI mean for reference to check for average RSI
    rsi_mean = rsi_ds['mean']
    if (curr_rsi < rsi_mean):
        rsi_avg = 'below average'
        rsi_gscore += 2
        if (rsi_direction != 'falling'):
            rsi_gscore += 2
    elif (curr_rsi > rsi_mean):
        rsi_avg = 'above average'
        rsi_gscore -= 4
    else:
        rsi_avg = 'average'

    rsi_max_score += 4

    #Higher weights when oversold or overbought
    #Score is calculated after checking percentile for number of days oversold/overbought
    if (curr_rsi <= RSI_OVERSOLD_LEVEL):
        rsi_lvl = 'oversold'
    elif (curr_rsi >= RSI_OVERBOUGHT_LEVEL):
        rsi_lvl = 'overbought'
    else:
        rsi_lvl = 'normal'

    curr_oversold_count = 0
    curr_overbought_count = 0

    avg_oversold_days = 0

    rsi_os_count_series = pd.Series(np.nan, pd.RangeIndex(rsi_len))
    rsi_os_count_index = 0

    rsi_ob_count_series = pd.Series(np.nan, pd.RangeIndex(rsi_len))
    rsi_ob_count_index = 0

    #Get oversold and overbought number of days stats
    for i in range(rsi_len):
        if (rsi[i] <= RSI_OVERSOLD_LEVEL):
            curr_oversold_count += 1
            rsi_os_count_series[rsi_os_count_index] = curr_oversold_count

        if (rsi[i] >= RSI_OVERBOUGHT_LEVEL):
            curr_overbought_count += 1
            rsi_ob_count_series[rsi_ob_count_index] = curr_overbought_count

        if ((rsi[i] > RSI_OVERSOLD_LEVEL) and (curr_oversold_count > 0)):
            rsi_os_count_index += 1
            curr_oversold_count = 0

        if ((rsi[i] < RSI_OVERBOUGHT_LEVEL) and (curr_overbought_count > 0)):
            rsi_ob_count_index += 1
            curr_overbought_count = 0


    rsi_os_count_series = rsi_os_count_series.dropna()
    rsi_os_count_series = rsi_os_count_series.sort_values()

    rsi_ob_count_series = rsi_ob_count_series.dropna()
    rsi_ob_count_series = rsi_ob_count_series.sort_values()


    if (curr_oversold_count > 0):
        #Get percentile values for oversold days count
        bp, mp, tp = rsi_os_count_series.quantile([0.25, 0.5, 0.75])

        if (curr_oversold_count < mp):
            curr_count_quantile_str = 'oversold day-count in bottom quantile'

        if (curr_oversold_count >= bp):
            rsi_gscore += 1

        if (curr_oversold_count >= mp):
            rsi_gscore += 2
            curr_count_quantile_str = 'oversold day-count in middle quantile'

        if (curr_oversold_count >= tp):
            rsi_gscore += 3
            curr_count_quantile_str = 'oversold day-count in top quantile'

    elif (curr_overbought_count > 0):
        #Get percentile values for overbought days count
        bp, mp, tp = rsi_ob_count_series.quantile([0.25, 0.5, 0.75])

        if (curr_overbought_count < mp):
            curr_count_quantile_str = 'overbought day-count in bottom quantile'

        if (curr_overbought_count >= bp):
            rsi_gscore -= 1

        if (curr_overbought_count >= mp):
            rsi_gscore -= 2
            curr_count_quantile_str = 'overbought day-count in middle quantile'

        if (curr_overbought_count >= tp):
            rsi_gscore -= 3
            curr_count_quantile_str = 'overbought day-count in top quantile'

    rsi_max_score += 6

    rsi_grec = f'rsi_gscore:{rsi_gscore}/{rsi_max_score}'
    rsi_signals = f'rsi: {rsi_avg},{rsi_lvl},{rsi_direction},{curr_count_quantile_str},{rsi_grec}'

    #Return RSI data in a dataframe for plotting charts
    rsi_df = pd.DataFrame({'RSI': rsi})

    return rsi_df, rsi_gscore, rsi_max_score, rsi_signals



# Author: Salyl Bhagwat, Gammath Works
# Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works
# All Rights Reserved
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__author__ = 'Salyl Bhagwat'
__copyright__ = 'Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works'

from pathlib import Path
import urllib.request
import sys


def get_stocktwits_ticker_info(tsymbol, path):

    STOCKTWITS_TICKER_ADDR = 'https://stocktwits.com'

    url = f'{STOCKTWITS_TICKER_ADDR}/symbol/{tsymbol}'

    st_ticker_page_html = ''

    if not path.exists():
        path.mkdir(parents=True, exist_ok=True)

    try:
        with urllib.request.urlopen(url) as response:
            page = response.read()
            html_page = f'{page}'
            #Save the page for reference
            f = open(path / f'{tsymbol}_st_page.html', 'w')
            f.write(html_page)
            f.close()
    except:
        raise RuntimeError('stocktwits html page')

    return




# Author: Salyl Bhagwat, Gammath Works
# Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works
# All Rights Reserved
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__author__ = 'Salyl Bhagwat'
__copyright__ = 'Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works'

import yfinance as yf
import pandas as pd
from datetime import datetime
from pathlib import Path
from talib import RSI, BBANDS, MACD, MFI, STOCH
from matplotlib import pyplot as plt
try:
    from gammath_spot import gammath_price_signals as gps
    from gammath_spot import gammath_rsi_signals as grs
    from gammath_spot import gammath_bb_signals as gbbs
    from gammath_spot import gammath_mfi_signals as gms
    from gammath_spot import gammath_stoch_signals as gss
    from gammath_spot import gammath_macd_signals as gmacd
    from gammath_spot import gammath_stcktwts_signals as gstw
    from gammath_spot import gammath_kf_signals as gkf
    from gammath_spot import gammath_options_signals as gos
    from gammath_spot import gammath_pe_signals as gpes
    from gammath_spot import gammath_peg_signals as gpeg
    from gammath_spot import gammath_beta_signals as gbeta
    from gammath_spot import gammath_ihp_signals as gihp
    from gammath_spot import gammath_inshp_signals as ginshp
    from gammath_spot import gammath_qbs_signals as gqbs
    from gammath_spot import gammath_pbr_signals as gpbrs
    from gammath_spot import gammath_reco_signals as greco
    from gammath_spot import gammath_ols_signals as gols
    from gammath_spot import gammath_get_stocks_events_data as gge
    from gammath_spot import gammath_score_signals as gscsi
    from gammath_spot import gammath_si_charts as gsc
except:
    import gammath_price_signals as gps
    import gammath_rsi_signals as grs
    import gammath_bb_signals as gbbs
    import gammath_mfi_signals as gms
    import gammath_stoch_signals as gss
    import gammath_macd_signals as gmacd
    import gammath_stcktwts_signals as gstw
    import gammath_kf_signals as gkf
    import gammath_options_signals as gos
    import gammath_pe_signals as gpes
    import gammath_peg_signals as gpeg
    import gammath_beta_signals as gbeta
    import gammath_ihp_signals as gihp
    import gammath_inshp_signals as ginshp
    import gammath_qbs_signals as gqbs
    import gammath_pbr_signals as gpbrs
    import gammath_reco_signals as greco
    import gammath_ols_signals as gols
    import gammath_get_stocks_events_data as gge
    import gammath_score_signals as gscsi
    import gammath_si_charts as gsc

import sys
import time
import os
import numpy as np

class GSA:

    def __init__(self):
        self.Tickers_dir = Path('tickers')
        self.overall_gscore = 0
        self.overall_max_score = 0
        self.reco_signals_exist = False
        self.note = 'Notes: None'

    def do_stock_analysis_and_compute_score(self, tsymbol):

        MIN_TRADING_DAYS_PER_YEAR = 249

        path = self.Tickers_dir / f'{tsymbol}'

        try:
            #Read Stock summary info into DataFrame.
            df_summ = pd.read_csv(path / f'{tsymbol}_summary.csv')
        except:
            print('\nERROR: Stock summary file not found for symbol ', tsymbol)
            return

        try:
            MIN_TRADING_DAYS_FOR_5_YEARS = (MIN_TRADING_DAYS_PER_YEAR*5)

            #Read CSV into DataFrame.
            df = pd.read_csv(path / f'{tsymbol}_history.csv')
            stock_history_len = len(df)

            #Analyze only if we have at least 5Y price data
            if (stock_history_len < MIN_TRADING_DAYS_FOR_5_YEARS):
                print(f'History doesn\'t have 5Y worth of data. Len: {stock_history_len} for {tsymbol}')
                raise RuntimeError('Not enough price history data')

            #Sometimes the price history doesn't have today's data.
            #Following is the workaround to detect that
            #This will show a note as an FYI (also if you ran it before market opens or on a holiday
            dt = time.strftime('%x').split('/')
            df_ld = df.Date[stock_history_len-1]
            df_ld = df_ld.split('-')
            if ((int(dt[0]) != int(df_ld[1])) or (int(dt[1]) != int(df_ld[2]))):
                raise ValueError('Stale price data')
        except ValueError:
            print('\nERROR: Stale price data for ', tsymbol)
            self.note = 'Note: NO_PRICE_DATA_FROM_TODAY'
        except:
            print('\nERROR: Stock price history error for ', tsymbol)
            return

        #Generate and get signals based on analyst recommendation
        try:
            reco_gscore = 0
            reco_max_score = 0 #In case there is an exception, we should do fundamental analysis
            reco_signals = ''
            #Reco signals
            reco_gscore, reco_max_score, reco_signals = greco.get_reco_signals(tsymbol, path)

            self.overall_gscore += reco_gscore
            self.overall_max_score += reco_max_score
            self.reco_signals_exist = ((reco_gscore != 0) and (reco_max_score != 0))
        except RuntimeError:
            print('\nERROR: while getting reco signals for ', tsymbol, ': ', sys.exc_info()[0])

        #Generate and get signals based on options activity
        try:
            options_gscore = 0
            options_max_score = 10
            options_signals = ''
            self.overall_max_score += options_max_score

            #Options signals
            options_gscore, options_max_score, options_signals = gos.get_options_signals(tsymbol, path, df.Close[len(df)-1], df_summ)

            self.overall_gscore += options_gscore

        except:
            print('\nERROR: while getting options signals for ', tsymbol, ': ', sys.exc_info()[0])

        #Generate and get signals based on fundamental analysis (PE, PEG, Beta, PBR, Quarterly balancesheet, Institutional holdings). Use it for scoring only of analyst recommendation don't exist for us
        try:
            pe_gscore = 0
            pe_max_score = 2
            pe_signals = ''

            if not self.reco_signals_exist:
                self.overall_max_score += pe_max_score

            #PE signals
            pe_gscore, pe_max_score, pe_signals = gpes.get_pe_signals(tsymbol, df_summ)
            if not self.reco_signals_exist:
                self.overall_gscore += pe_gscore

        except ValueError:
            print('\nERROR: while getting PE signals for ', tsymbol, ': ', sys.exc_info()[0])

        try:
            peg_gscore = 0
            peg_max_score = 1
            peg_signals = ''

            if not self.reco_signals_exist:
                self.overall_max_score += peg_max_score

            #PEG signals
            peg_gscore, peg_max_score, peg_signals = gpeg.get_peg_signals(tsymbol, df_summ)

            if not self.reco_signals_exist:
                self.overall_gscore += peg_gscore

        except ValueError:
            print('\nERROR: while getting PEG signals for ', tsymbol, ': ', sys.exc_info()[0])

        try:
            beta_gscore = 0
            beta_max_score = 1
            beta_signals = ''

            if not self.reco_signals_exist:
                self.overall_max_score += beta_max_score

            #Beta signals
            beta_gscore, beta_max_score, beta_signals = gbeta.get_beta_signals(tsymbol, df_summ)

            if not self.reco_signals_exist:
                self.overall_gscore += beta_gscore

        except ValueError:
            print('\nERROR: while getting beta signals for ', tsymbol, ': ', sys.exc_info()[0])

        try:
            pbr_gscore = 0
            pbr_max_score = 1
            pbr_signals = ''

            if not self.reco_signals_exist:
                self.overall_max_score += pbr_max_score

            #PBR signals
            pbr_gscore, pbr_max_score, pbr_signals = gpbrs.get_pbr_signals(tsymbol, df_summ)

            if not self.reco_signals_exist:
                self.overall_gscore += pbr_gscore

        except RuntimeError:
            print('\nERROR: while generating PBR signals for ', tsymbol, ': ', sys.exc_info()[0])
        except ValueError:
            print('\nERROR: while getting PBR signals for ', tsymbol, ': ', sys.exc_info()[0])

        try:
            qbs_gscore = 0
            qbs_max_score = 4
            qbs_signals = ''

            if not self.reco_signals_exist:
                self.overall_max_score += qbs_max_score

            #Quarterly Balance sheet signals
            qbs_gscore, qbs_max_score, qbs_signals = gqbs.get_qbs_signals(tsymbol, path, df_summ)

            if not self.reco_signals_exist:
                self.overall_gscore += qbs_gscore

        except ValueError:
            print('\nERROR: while getting QBS signals for ', tsymbol, ': ', sys.exc_info()[0])

        try:
            ihp_gscore = 0
            ihp_max_score = 1
            ihp_signals = ''

            if not self.reco_signals_exist:
                self.overall_max_score += ihp_max_score

            #Institutional Holders Percentage signals
            ihp_gscore, ihp_max_score, ihp_signals = gihp.get_ihp_signals(tsymbol, df_summ)

            if not self.reco_signals_exist:
                self.overall_gscore += ihp_gscore

        except ValueError:
            print('\nERROR: while getting ihp signals for ', tsymbol, ': ', sys.exc_info()[0])

        try:
            inshp_gscore = 0
            inshp_max_score = 1
            inshp_signals = ''

            #Insider Holders Percentage signals
            inshp_gscore, inshp_max_score, inshp_signals = ginshp.get_inshp_signals(tsymbol, df_summ)
        except ValueError:
            print('\nERROR: while getting inshp signals for ', tsymbol, ': ', sys.exc_info()[0])

        #Generate and get signals based on Price history
        try:
            price_gscore = 0
            price_max_score = 10
            price_signals = ''
            self.overall_max_score += price_max_score

            #Price signals
            price_gscore, price_max_score, price_signals = gps.get_price_signals(tsymbol, df, df_summ)

            self.overall_gscore += price_gscore

        except ValueError:
            print('\nERROR: generating price signals for ', tsymbol, ': ', sys.exc_info()[0])

        #Generate and get signals based on Relative Strength Index Indicator
        try:
            rsi_gscore = 0
            rsi_max_score = 10
            rsi_signals = ''
            self.overall_max_score += rsi_max_score

            #Relative Strength Index signals
            rsi_df, rsi_gscore, rsi_max_score, rsi_signals = grs.get_rsi_signals(tsymbol, df, path)

            self.overall_gscore += rsi_gscore

        except RuntimeError:
            print('\nERROR: generating RSI data for ', tsymbol, ': ', sys.exc_info()[0])
        except ValueError:
            print('\nERROR: generating signals from RSI data ', tsymbol, ': ', sys.exc_info()[0])

        #Generate and get signals based on Money Flow Index Indicator
        try:
            mfi_gscore = 0
            mfi_max_score = 10
            mfi_signals = ''
            self.overall_max_score += mfi_max_score

            #MFI signals
            mfi_df, mfi_gscore, mfi_max_score, mfi_signals = gms.get_mfi_signals(tsymbol, df, path)

            self.overall_gscore += mfi_gscore

        except RuntimeError:
            print('\nERROR: generating MFI data for ', tsymbol, ': ', sys.exc_info()[0])
        except ValueError:
            print('\nERROR: generating signals from MFI data ', tsymbol, ': ', sys.exc_info()[0])

        #Generate and get signals based on Stochastic Indicator
        try:
            stoch_gscore = 0
            stoch_max_score = 5
            stoch_slow_signals = ''
            self.overall_max_score += stoch_max_score

            #Stochastic slow signals
            stoch_df, stoch_gscore, stoch_max_score, stoch_slow_signals = gss.get_stochastics_slow_signals(tsymbol, df)

            self.overall_gscore += stoch_gscore

        except RuntimeError:
            print('\nERROR: generating stochastics data for ', tsymbol, ': ', sys.exc_info()[0])
        except ValueError:
            print('\nERROR: generating signals from stochastics data ', tsymbol, ': ', sys.exc_info()[0])

        #Generate and get signals based on Bollinger Bands Indicator
        try:
            bb_gscore = 0
            bb_max_score = 10
            bb_signals = ''
            self.overall_max_score += bb_max_score

            #Bollinger bands signals
            bb_df, bb_gscore, bb_max_score, bb_signals = gbbs.get_bollinger_bands_signals(tsymbol, df, path)

            self.overall_gscore += bb_gscore

        except RuntimeError:
            print('\nERROR: generating Bollinger Bands for ', tsymbol, ': ', sys.exc_info()[0])
        except ValueError:
            print('\nERROR: generating signals from Bollinger Bands for ', tsymbol, ': ', sys.exc_info()[0])

        #Generate and get signals based on Moving Average Convergence/Divergence Indicator
        try:
            macd_gscore = 0
            macd_max_score = 10
            macd_signals = ''
            self.overall_max_score += macd_max_score

            #MACD signals
            macd_df, macd_gscore, macd_max_score, macd_signals = gmacd.get_macd_signals(tsymbol, df, path)

            self.overall_gscore += macd_gscore

        except RuntimeError:
            print('\nERROR: generating MACD data for ', tsymbol, ': ', sys.exc_info()[0])
        except ValueError:
            print('\nERROR: generating signals from MACD data ', tsymbol, ': ', sys.exc_info()[0])

        #Generate and get signals based on Kalman Filter state means
        try:
            kf_gscore = 0
            kf_max_score = 10
            kf_signals = ''
            self.overall_max_score += kf_max_score

            #Kalman Filter signals
            kf_df, kf_gscore, kf_max_score, kf_signals = gkf.get_kf_state_means(tsymbol, df)

            self.overall_gscore += kf_gscore

        except RuntimeError:
            print('\nERROR: generating Kalman filter data for ', tsymbol, ': ', sys.exc_info()[0])
        except ValueError:
            print('\nERROR: generating signals from Kalman filter data ', tsymbol, ': ', sys.exc_info()[0])

        #Generate and get signals based on Ordinary Least Squares line
        try:
            ols_gscore = 0
            ols_max_score = 10
            ols_signals = ''
            self.overall_max_score += ols_max_score

            #Ordinary Least Squares line signals
            ols_df, ols_gscore, ols_max_score, ols_signals = gols.get_ols_signals(tsymbol, df, path)

            self.overall_gscore += ols_gscore

        except RuntimeError:
            print('\nERROR: generating OLS data for ', tsymbol, ': ', sys.exc_info()[0])
        except ValueError:
            print('\nERROR: generating signals from OLS data ', tsymbol, ': ', sys.exc_info()[0])

        #Generate and get signals based on Stocktwits sentiment and volume change
        try:
            st_gscore = 0
            st_max_score = 5
            st_signals = ''
            self.overall_max_score += st_max_score

            #StockTwits signals
            st_gscore, st_max_score, st_signals = gstw.get_stocktwits_signals(tsymbol, path)

            self.overall_gscore += st_gscore

        except RuntimeError:
            print('\nERROR: while getting stocktwits signals for ', tsymbol, ': ', sys.exc_info()[0])

        #Get info on earnings etc. to be aware before making buy/sell decision
        try:
            #Get events info
            events_info = gge.get_events_info(tsymbol, path)
        except:
            print('\nERROR: while getting events info for ', tsymbol, ': ', sys.exc_info()[0])
            events_info = ''

        #Augment all signals for saving in a file
        overall_signals = f'{price_signals}\n{rsi_signals}\n{bb_signals}\n{macd_signals}\n{kf_signals}\n{ols_signals}\n{mfi_signals}\n{stoch_slow_signals}\n{options_signals}\n{pe_signals}\n{peg_signals}\n{beta_signals}\n{ihp_signals}\n{inshp_signals}\n{qbs_signals}\n{pbr_signals}\n{reco_signals}\n{st_signals}\n{events_info}\n{self.note}'

        #Compute final score then save scores and signals
        try:
            gscsi.compute_final_score_and_save_signals(tsymbol, path, self.overall_gscore, self.overall_max_score, overall_signals)
        except:
            print('\nERROR: while computing final score and saving signals for ', tsymbol, ': ', sys.exc_info()[0])

        #Plot and save charts for reference
        try:
            gsc.plot_and_save_charts(tsymbol, bb_df, rsi_df, mfi_df, macd_df, stoch_df, kf_df, ols_df)
        except:
            print('\nERROR: while drawing and saving charts for ', tsymbol, ': ', sys.exc_info()[0])



# Author: Salyl Bhagwat, Gammath Works
# Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works
# All Rights Reserved
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__author__ = 'Salyl Bhagwat'
__copyright__ = 'Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works'

from pathlib import Path
import pandas as pd
import numpy as np

def get_qbs_signals(tsymbol, path, df_summ):

    cash = 0
    lti = 0
    sti = 0
    cash_earned_last_one_year = 0
    possible_remaining_cash = 0
    sequity = 0
    dtcr = 0

    qbs_gscore = 0
    qbs_max_score = 0

    cr = 0
    qr = 0

    #Check for current ratio for debt obligation readiness
    try:
        cr = df_summ['currentRatio'][0]
        if (np.isnan(cr)):
            cr_string = 'No current ratio data'
        else:
            cr = round(cr, 3)
            cr_string = f'{cr}'
    except:
        cr_string = 'No current ratio data'

    #Check for quick ratio for debt obligation ease
    try:
        qr = df_summ['quickRatio'][0]
        if (np.isnan(qr)):
            qr_string = 'No quick ratio data'
        else:
            qr = round(qr, 3)
            qr_string = f'{qr}'
    except:
        qr_string = 'No quick ratio data'

    file_exists = (path / f'{tsymbol}_qbs.csv').exists()

    #Check if file exists and is it from another day
    if file_exists:

        #Read Quarterly balance sheet CSV
        df = pd.read_csv(path / f'{tsymbol}_qbs.csv', index_col='Unnamed: 0')

        if (len(df) == 0):
            raise ValueError('QBS dataframe empty')
        else:

            #Get the most recent quarter date
            mrqd = df.columns[0]

            try:
                #Cash position at the end of last reported quarter
                cash = df[mrqd]['Cash']
                earnings_file_exists = (path / f'{tsymbol}_qe.csv').exists()
                if (earnings_file_exists):
                    dfe = pd.read_csv(path / f'{tsymbol}_qe.csv', index_col='Unnamed: 0')
                    try:
                        dfe_len = len(dfe)
                        if (dfe_len > 0):
                            if (dfe_len >= 4):
                                #Get the sum for last 4 quarters
                                cash_earned_last_one_year = dfe.Earnings[(dfe_len-4):].sum()
                    except:
                        cash_earned_last_one_year = 0
            except:
                cash = 0

            try:
                sti = df[mrqd]['Short Term Investments']
            except:
                sti = 0

            try:
                lti = df[mrqd]['Long Term Investments']
            except:
                lti = 0

            try:
                #Total shareholder equity
                sequity = df[mrqd]['Total Stockholder Equity']
            except:
                sequity = 0

            try:
                #Long term debt
                ldebt = df[mrqd]['Long Term Debt']
            except:
                ldebt = 0

            #Debt to capital ratio. TBD: Need to revisit the formula
            if ((ldebt > 0) and (sequity > 0)):
                dtcr = round((ldebt / (ldebt + sequity)), 3)

            #Check if current ratio is >= 1
            if (cr >= 1):
                qbs_gscore += 1
            else:
                qbs_gscore -= 1

            qbs_max_score += 1

            #Add remaining cash to get an idea of cash position
            possible_remaining_cash += (cash + sti + lti)

            #Check if quick ratio >= 1
            if (qr >= 1):
                qbs_gscore += 1

            qbs_max_score += 1

            if (sequity > 0):
                qbs_gscore += 1
            else:
                qbs_gscore -= 1

            qbs_max_score += 1

            if (ldebt > 0) and (dtcr <= 0.5):
                qbs_gscore += 1
            else:
                qbs_gscore -= 1

            #Max score from debt data
            qbs_max_score += 1
    else:
        #This will show 0/4 when no balance sheet data
        raise ValueError('QBS file doesn\'t exist')

    qbs_grec = f'qbs_gscore:{qbs_gscore}/{qbs_max_score}'

    qbs_signals = f'qbs:dtcr:{dtcr},cr:{cr_string},qr:{qr_string},{qbs_grec}'

    return qbs_gscore, qbs_max_score, qbs_signals



# Author: Salyl Bhagwat, Gammath Works
# Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works
# All Rights Reserved
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__author__ = 'Salyl Bhagwat'
__copyright__ = 'Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works'

import yfinance as yf
from pathlib import Path
import bisect
from datetime import datetime
import pandas as pd
import time
import os
try:
    from gammath_spot import gammath_utils as gut
except:
    import gammath_utils as gut

def get_options_data(tsymbol, ticker, path):

    #Get stock info summary from the internet

    if (len(tsymbol) == 0):
        raise ValueError('Invalid symbol')

    try:

        #Check if file exists and is it from another day
        file_exists = (path / f'{tsymbol}_options_dates.csv').exists()

        if (file_exists):
            #Check if it is latest
            fstat = os.stat(path / f'{tsymbol}_options_dates.csv')
            dont_need_fetch = gut.check_if_same_day(fstat)
        else:
            dont_need_fetch = False

        if (dont_need_fetch):
            #Get the latest options expiry date
            dates_df = pd.read_csv(path / f'{tsymbol}_options_dates.csv', index_col='Unnamed: 0')

            option_date = dates_df.loc[0][0]
        else:
            #Read fresh data
            option_dates = ticker.options
            if (len(option_dates)):
                opt_dates_ds = pd.Series(option_dates)
                opt_dates_ds.to_csv(path / f'{tsymbol}_options_dates.csv')

                option_date = option_dates[0]
            else:
                raise RuntimeError('No option expiry dates')

        #Check if file exists and is it from another day
        file_exists = (path / f'{tsymbol}_call_{option_date}.csv').exists()

        if file_exists:
            #Check if it is latest
            fstat = os.stat(path / f'{tsymbol}_call_{option_date}.csv')
            dont_need_fetch = gut.check_if_same_day(fstat)
        else:
            dont_need_fetch = False

        #We only need to read options data once a day
        if (not dont_need_fetch):
            options = ticker.option_chain(option_date)
            df_calls = options.calls.sort_values('strike')

            #Save the calls sorted by strike price
            df_calls.to_csv(path / f'{tsymbol}_call_{option_date}.csv')

            df_puts = options.puts.sort_values('strike')

            #Save the puts sorted by strike price
            df_puts.to_csv(path / f'{tsymbol}_put_{option_date}.csv')
    except:
        raise RuntimeError('No option data')

    return




# Author: Salyl Bhagwat, Gammath Works
# Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works
# All Rights Reserved
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__author__ = 'Salyl Bhagwat'
__copyright__ = 'Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works'

from pathlib import Path
import sys

def compute_final_score_and_save_signals(tsymbol, path, overall_gscore, overall_max_score, overall_signals):

    overall_grec = f'overall_gscore:{overall_gscore}/{overall_max_score}'

    if (overall_max_score != 0):
        final_gscore = round((int(overall_gscore)/int(overall_max_score)), 5)
    else:
        final_gscore = 0

    final_gscore_rec = f'final_gscore:{final_gscore}'

    try:
        f = open(path / 'signal.txt', 'w')
    except:
        print('\nERROR: opening signal file for ', tsymbol, ': ', sys.exc_info()[0])
    else:
        f.write(f'{overall_signals}\n{overall_grec}\n{final_gscore_rec}\n')
        f.close()



# Author: Salyl Bhagwat, Gammath Works
# Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works
# All Rights Reserved
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__author__ = 'Salyl Bhagwat'
__copyright__ = 'Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works'

from pathlib import Path
import pandas as pd
import numpy as np


def get_pe_signals(tsymbol, df_summ):

    Tickers_dir = Path('./tickers')

    path = Tickers_dir
    if not path.exists():
        path.mkdir(parents=True, exist_ok=True)

    pe_gscore = 0
    pe_max_score = 2

    if (path / 'SP500_SEC_PES.csv').exists():
        df_sp = pd.read_csv(path / 'SP500_SEC_PES.csv')

    tpe = 0
    fpe = 0

    try:
        tpe = df_summ['trailingPE'][0]
        if (np.isnan(tpe)):
            tpe_string = 'No TPE data'
        else:
            tpe = round(tpe, 3)
            tpe_string = f'{tpe}'
    except:
        tpe_string = 'No TPE data'

    try:
        fpe = df_summ['forwardPE'][0]
        if (np.isnan(fpe)):
            fpe_string = 'No FPE data'
        else:
            fpe = round(fpe, 3)
            fpe_string = f'{fpe}'
    except:
        fpe_string = 'No FPE data'


    # For now just check if FPE is less or more than TPE
    # If forward PE is less than trailing PE then view this as a +ve sign
    if ((fpe > 0) and (tpe > 0)):
        if (fpe < tpe):
            pe_gscore += 2
        else:
            pe_gscore -= 2
    else:
        pe_gscore -= 2


    avg_tpe = 0
    avg_fpe = 0

    len_df_sp = len(df_sp)

    for i in range(len_df_sp):
        if (df_sp['Symbol'][i] == tsymbol):
            try:
                avg_tpe = df_sp['LS_AVG_TPE'][i]
                if (np.isnan(avg_tpe)):
                    avg_tpe_string = 'No Avg TPE data'
                else:
                    avg_tpe = round(avg_tpe, 3)
                    avg_tpe_string = f'{avg_tpe}'
            except:
                avg_tpe_string = 'No Avg TPE data'

            try:
                avg_fpe = df_sp['LS_AVG_FPE'][i]
                if (np.isnan(avg_fpe)):
                    avg_fpe_string = 'No Avg FPE data'
                else:
                    avg_fpe = round(avg_fpe, 3)
                    avg_fpe_string = f'{avg_fpe}'
            except:
                avg_fpe_string = 'No Avg FPE data'

            # Avg TPE and Avg FPE data needs to be authentic before we can factor it into score computation.
            # Currently, this avg is based on very limited data and also does not include sub-sectors
            # For now just use this for logging for later manual and rough comparison
            break

    if (i == (len_df_sp-1)):
        avg_tpe_string = 'No Avg TPE data'
        avg_fpe_string = 'No Avg FPE data'


    pe_grec = f'pe_gscore:{pe_gscore}/{pe_max_score}'

    pe_signals = f'PE: TPE:{tpe_string},ATPE:{avg_tpe_string},FPE:{fpe_string},AFPE:{avg_fpe_string},{pe_grec}'

    return pe_gscore, pe_max_score, pe_signals



# Author: Salyl Bhagwat, Gammath Works
# Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works
# All Rights Reserved
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__author__ = 'Salyl Bhagwat'
__copyright__ = 'Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works'

import pandas as pd
from pathlib import Path
from talib import MFI
import numpy as np


def get_mfi_signals(tsymbol, df, path):

    MFI_TIME_PERIOD = 14
    MFI_OVERSOLD_LEVEL = 20
    MFI_OVERBOUGHT_LEVEL = 80

    mfi_gscore = 0
    mfi_max_score = 0
    mfi_signals = ''

    mfi_avg = ''
    mfi_dir = ''
    mfi_lvl = ''

    #Using MFI only to look for probable price direction reversal indicator
    mfi_indicator = ''

    prices = df.Close
    prices_len = len(prices)
    if (prices_len <= 0):
        raise ValueError('Invalid Price data for generating MFI')
    else:
        lp = prices[prices_len-1]
        lpm1 = prices[prices_len-2]

    try:
        mfi = MFI(df.High, df.Low, df.Close, df.Volume, timeperiod=MFI_TIME_PERIOD)
        mfi_len = len(mfi)
        mfi_ds = mfi.describe()
        mfi_mean = mfi_ds['mean']
    except:
        raise RuntimeError('MFI data generation failed')

    #Get current price direction
    if (lp < lpm1):
        price_dir = 'falling'
    elif (lp > lpm1):
        price_dir = 'rising'
    else:
        price_dir = 'direction_unclear'

    if (mfi_len > 0):
        curr_mfi = mfi[mfi_len-1]
        lm1_mfi = mfi[mfi_len-2]

        #Get the current MFI level compared to the mean
        if (curr_mfi < mfi_mean):
            mfi_avg = 'below average'
            mfi_gscore += 3
        elif (curr_mfi > mfi_mean):
            mfi_avg = 'above average'
            mfi_gscore -= 3
        else:
            mfi_avg = 'average'

        mfi_max_score += 3

        if (curr_mfi >= MFI_OVERBOUGHT_LEVEL):
            mfi_lvl = 'overbought'
            mfi_gscore -= 4
        elif (curr_mfi <= MFI_OVERSOLD_LEVEL):
            mfi_lvl = 'oversold'
            mfi_gscore += 4
        else:
            mfi_lvl = ''

        mfi_max_score += 4

        if (curr_mfi < lm1_mfi):
            mfi_dir = 'falling'
            #Look for reversal when overbought/oversold
            if (curr_mfi >= MFI_OVERBOUGHT_LEVEL):
                if (price_dir == 'rising'):
                    mfi_indicator = 'Price could start FALLING'
                    mfi_gscore -= 3
        elif (curr_mfi > lm1_mfi):
            mfi_dir = 'rising'
            #Look for reversal when overbought/oversold
            if (curr_mfi <= MFI_OVERSOLD_LEVEL):
                if (price_dir == 'falling'):
                    mfi_indicator = 'Price could start RISING'
                    mfi_gscore += 3
        else:
            mfi_dir = 'flat'

        mfi_max_score += 3
    else:
        raise ValueError('Invalid MFI data length ')

    mfi_grec = f'mfi_gscore:{mfi_gscore}/{mfi_max_score}'

    mfi_signals = f'mfi:{mfi_avg},{mfi_dir},{mfi_lvl},{mfi_grec},{mfi_indicator}'

    #Return RSI data in a dataframe for plotting charts
    mfi_df = pd.DataFrame({'MFI': mfi})

    return mfi_df, mfi_gscore, mfi_max_score, mfi_signals




# Author: Salyl Bhagwat, Gammath Works
# Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works
# All Rights Reserved
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__author__ = 'Salyl Bhagwat'
__copyright__ = 'Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works'

import yfinance as yf
from pathlib import Path
import pandas as pd
import sys
import time
import os

def get_ticker_summary(tsymbol, ticker, path):

    #Get stock info summary from the internet

    if (len(tsymbol) == 0):
        raise ValueError('Invalid symbol')

    Tickers_dir = Path('tickers')

    #Check if file exists and is it from another day
    file_exists = (path / f'{tsymbol}_summary.csv').exists()

    if file_exists:
        fstat = os.stat(path / f'{tsymbol}_summary.csv')
        fct_time = time.ctime(fstat.st_ctime).split(' ')
        if (fct_time[2] == ''):
            fct_date_index = 3
        else:
            fct_date_index = 2

        fct_date = int(fct_time[fct_date_index])
        dt = time.strftime('%x').split('/')
        dt_date = int(dt[1])

        if (fct_date == dt_date):
            dont_need_fetch = True
        else:
            dont_need_fetch = False
    else:
        dont_need_fetch = False

    #Check if we need to get new summary info
    if (dont_need_fetch):
        return
    else:
        try:
            stock_summary = ticker.info
        except:
            raise ValueError('Stock summary not found')

    #Extract the items of interest
    #trailingPE
    #forwardPE
    #fiftyTwoWeekHigh
    #fiftyTwoWeekLow
    #fiftyDayAverage
    #twoHundredDayAverage
    #shortRatio
    #pegRatio
    #currentRatio
    #quickRatio
    #beta
    #heldPercentInstitutions
    #heldPercentInsiders
    #priceToBook
    #state
    #country
    #regularMarketPrice
    #marketCap

    try:
        trailingPE = stock_summary['trailingPE']
    except:
        trailingPE = 0

    try:
        forwardPE = stock_summary['forwardPE']
    except:
        forwardPE = 0

    try:
        fiftyTwoWeekHigh = stock_summary['fiftyTwoWeekHigh']
    except:
        fiftyTwoWeekHigh = 0

    try:
        fiftyTwoWeekLow = stock_summary['fiftyTwoWeekLow']
    except:
        fiftyTwoWeekLow = 0

    try:
        fiftyDayAverage = stock_summary['fiftyDayAverage']
    except:
        fiftyDayAverage = 0

    try:
        twoHundredDayAverage = stock_summary['twoHundredDayAverage']
    except:
        twoHundredDayAverage = 0

    try:
        shortRatio = stock_summary['shortRatio']
    except:
        shortRatio = 0

    try:
        pegRatio = stock_summary['pegRatio']
    except:
        pegRatio = 0

    try:
        currentRatio = stock_summary['currentRatio']
    except:
        currentRatio = 0

    try:
        quickRatio = stock_summary['quickRatio']
    except:
        quickRatio = 0

    try:
        beta = stock_summary['beta']
    except:
        beta = 0

    try:
        heldPercentInstitutions = stock_summary['heldPercentInstitutions']
    except:
        heldPercentInstitutions = 0
    try:
        heldPercentInsiders = stock_summary['heldPercentInsiders']
    except:
        heldPercentInsiders = 0

    try:
        pbr = stock_summary['priceToBook']
    except:
        pbr = 0

    try:
        state = stock_summary['state']
    except:
        state = ''

    try:
        country = stock_summary['country']
    except:
        country = ''

    try:
        curr_price = stock_summary['regularMarketPrice']
    except:
        curr_price = 0

    try:
        mktcap = stock_summary['marketCap']
    except:
        mktcap = 0

    df = pd.DataFrame({'trailingPE': trailingPE, 'forwardPE': forwardPE, 'fiftyTwoWeekHigh': fiftyTwoWeekHigh, 'fiftyTwoWeekLow': fiftyTwoWeekLow, 'fiftyDayAverage': fiftyDayAverage, 'twoHundredDayAverage': twoHundredDayAverage, 'shortRatio': shortRatio, 'pegRatio': pegRatio, 'currentRatio': currentRatio, 'quickRatio': quickRatio, 'beta': beta, 'heldPercentInstitutions': heldPercentInstitutions, 'heldPercentInsiders': heldPercentInsiders, 'priceToBook': pbr, 'state': state, 'country': country, 'currentPrice': curr_price, 'marketCap': mktcap}, index=range(1))

    if not path.exists():
        path.mkdir(parents=True, exist_ok=True)

    #Save the history for reference and processing
    df.to_csv(path / f'{tsymbol}_summary.csv')

    return



# Author: Salyl Bhagwat, Gammath Works
# Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works
# All Rights Reserved
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__author__ = 'Salyl Bhagwat'
__copyright__ = 'Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works'

from pathlib import Path
import time
import re
import sys

STOCKTWITS_TICKER_ADDR = 'https://stocktwits.com'

def get_stocktwits_signals(tsymbol, path):

    st_gscore = 0
    st_max_score = 0
    sentiment_change = None
    volume_change = None

    #RE to extract sentiment change value
    pattern_for_sentiment_change = re.compile(r'("sentimentChange"):([-]*[0-9]*[.]*[0-9]*)')

    #RE to extract volume change value
    pattern_for_volume_change = re.compile(r'("volumeChange"):([-]*[0-9]*[.]*[0-9]*)')
    st_ticker_page_html = ''

    if not path.exists():
        path.mkdir(parents=True, exist_ok=True)

    try:
        #Read the saved page
        f = open(path / f'{tsymbol}_st_page.html')
        html_page = f.read()
        f.close()

        #Find the sentiment change score
        matched_string = pattern_for_sentiment_change.search(html_page)
        if (matched_string):
            kw, val = matched_string.groups()
            sentiment_change = val

        #Find the volume change score
        matched_string = pattern_for_volume_change.search(html_page)
        if (matched_string):
            kw, val = matched_string.groups()
            volume_change = val

        sts_change = 0
        stv_change = 0

        if (sentiment_change is not None):
            #Convert to the float type
            sts_change = float(sentiment_change)
            st_tw_sentiment_change = f'sentiment_change: {sentiment_change}'

        if (volume_change is not None):
            #Convert to the float type
            stv_change = float(volume_change)
            st_tw_volume_change = f'volume_change: {volume_change}'

        #Token score and information logging purpose
        if ((sts_change > 0) and (stv_change > 0)):
            st_gscore += 5
        elif (sts_change > 0):
            st_gscore += 2
        else:
            st_gscore -= 5

    except:
        raise RuntimeError('Stocktwits signal generation failed')

    st_max_score += 5

    st_grec = f'st_sv_gscore:{st_gscore}/{st_max_score}'

    st_signals = f'{st_grec}'

    return st_gscore, st_max_score, st_signals




# Author: Salyl Bhagwat, Gammath Works
# Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works
# All Rights Reserved
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__author__ = 'Salyl Bhagwat'
__copyright__ = 'Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works'

from pathlib import Path
import pandas as pd
import numpy as np

def get_beta_signals(tsymbol, df_summ):

    beta_gscore = 0
    beta_max_score = 0

    try:
        #Get the beta value from summary DF
        beta = df_summ['beta'][0]
        if (np.isnan(beta)):
            beta_string = 'No beta data'
        else:
            #round it off for taking less space when displaying
            beta = round(beta, 3)
            beta_string = f'{beta}'
            if (beta > 0):
                #Closer to 1 is near market and is better
                if (beta < 2):
                    beta_gscore += 1
                else:
                    beta_gscore -= 1
    except:
        beta_string = 'No beta data'

    beta_max_score += 1

    beta_grec = f'beta_gscore:{beta_gscore}/{beta_max_score}'

    beta_signals = f'BETA:{beta_string},{beta_grec}'

    return beta_gscore, beta_max_score, beta_signals



# Author: Salyl Bhagwat, Gammath Works
# Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works
# All Rights Reserved
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__author__ = 'Salyl Bhagwat'
__copyright__ = 'Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works'

import sys
import os
import time
from pathlib import Path
import pandas as pd
import re

def check_if_same_day(fstat):

    fct_time = time.ctime(fstat.st_ctime).split(' ')
    dt = time.strftime('%x').split('/')
    if (fct_time[2] == ''):
        fct_date_index = 3
    else:
        fct_date_index = 2

    fct_date = int(fct_time[fct_date_index])
    dt_date = int(dt[1])

    if (fct_date == dt_date):
        return True
    else:
        return False

class GUTILS:

    def __init__(self):
        self.Tickers_dir = Path('tickers')


    def get_sp500_list(self):

        sp500_list_url = f'https://en.wikipedia.org/wiki/List_of_S&P_500_companies'

        path = self.Tickers_dir
        if not path.exists():
            path.mkdir(parents=True, exist_ok=True)

        #Fetch the file only once a day
        dont_need_fetch = True

        try:
            #Get existing file
            file_exists = (path / f'SP500_list.csv').exists()

            #Check if file exists and is it from another day
            if file_exists:
                fstat = os.stat(path / f'SP500_list.csv')
                dont_need_fetch = check_if_same_day(fstat)
            else:
                #File doesn't exist/
                dont_need_fetch = False
        except:
            dont_need_fetch = False

        if (not dont_need_fetch):
            #Get S&P500 list from the internet.
            #Specify header (row to use to make column headers)
            #No need to get entire list of dataframes. We only need first dataframe
            sp500 = pd.read_html(sp500_list_url, header=0)[0]

            #Save the history for reference and processing
            sp500.to_csv(path / f'SP500_list.csv')

        return

    def aggregate_scores(self):

        #Get all the subdirs. Need to check for is_dir
        p = self.Tickers_dir

        #Somehow looks like os.is_dir isn't supported
        #Using pathlib/Path instead since is_dir is supported there
        subdirs = [x for x in p.iterdir() if x.is_dir()]

        pattern_for_final_gscore = re.compile(r'(final_gscore):([-]*[0-9]*[.]*[0-9]+)')

        #Pattern for note
        pattern_for_note = re.compile(r'(Note):([\s]*[A-Z]*[_]*[A-Z]*[_]*[A-Z]*[_]*[A-Z]*[_]*[A-Z]*[_]*[A-Z]*)')

        df_b = pd.DataFrame(columns=['Ticker', 'final_gscore', 'Note'], index=range(len(subdirs)))

        i = 0

        for subdir in subdirs:
            if subdir.exists():
                try:
                    f = open(subdir / 'signal.txt', 'r')
                    content = f.read()

                    matched_string = pattern_for_note.search(content)
                    if (matched_string):
                        kw, note = matched_string.groups()
                        df_b['Note'][i] = note
                    else:
                        df_b['Note'][i] = ''

                    matched_string = pattern_for_final_gscore.search(content)
                    if (matched_string):
                        kw, val = matched_string.groups()
                        df_b['Ticker'][i] = f'{subdir.name}'
                        df_b['final_gscore'][i] = float(val)
                        i += 1

                    f.close()
                except:
                    print('\nERROR: Getting stock signals for ', subdir.name, ': ', sys.exc_info()[0])

        df_b.sort_values('final_gscore').dropna(how='all').to_csv(self.Tickers_dir / 'overall_gscores.csv', index=False)


    def aggregate_pe_data(self):

        path = self.Tickers_dir
        if not path.exists():
            path.mkdir(parents=True, exist_ok=True)

        df = pd.read_csv(path / 'SP500_list.csv')

        #Need to calculate sector-average so rearrange
        df_sp = df.sort_values('GICS Sector')

        df_sp_len = len(df_sp)

        #Create new dataframe for holding trailing/forward PE and their respective sector averages
        df_pe = pd.DataFrame(columns=['TPE', 'FPE', 'LS_AVG_TPE', 'LS_AVG_FPE'], index=range(df_sp_len))

        i = 0
        #Get all symbols in list
        symbols = list(df_sp['Symbol'])

        for symbol in symbols:
            try:
                df_summ = pd.read_csv(path / f'{symbol}/{symbol}_summary.csv')
                tpe = df_summ['trailingPE'][0]
                fpe = df_summ['forwardPE'][0]

                #df_sp Symbols are arranged based on sectors so same order will be in df_pe
                df_pe['TPE'][i] = tpe
                df_pe['FPE'][i] = fpe

            except:
                df_pe['TPE'][i] = 0
                df_pe['FPE'][i] = 0

            i += 1

        #Extract unique sectors
        sectors = df_sp['GICS Sector'].drop_duplicates()
        sector_list = []

        #Calculate average and save for each sector; also save sectors
        i = 0
        new_tpe = 0
        new_fpe = 0

        sector_fields = list(df_sp['GICS Sector'])
        len_sector_fields = len(sector_fields)
        tpes = list(df_pe['TPE'])
        fpes = list(df_pe['FPE'])

        for sector in sectors:
            sector_list.append(sector)
            curr_sector_tpe = 0
            curr_sector_tpe_count = 0
            curr_sector_fpe = 0
            curr_sector_fpe_count = 0
            start_index = i
            while (sector_fields[i] == sector):
                new_tpe = tpes[i]
                new_fpe = fpes[i]
                i += 1

                if (new_tpe > 0):
                    curr_sector_tpe_count += 1
                    curr_sector_tpe += new_tpe

                if (new_fpe > 0):
                    curr_sector_fpe_count += 1
                    curr_sector_fpe += new_fpe

                if (i == len_sector_fields):
                    break

            end_index = i
            curr_sector_tpe_avg = 0
            curr_sector_fpe_avg = 0

            if (curr_sector_tpe_count):
                curr_sector_tpe_avg = curr_sector_tpe / curr_sector_tpe_count

            if (curr_sector_fpe_count):
                curr_sector_fpe_avg = curr_sector_fpe / curr_sector_fpe_count


            #Save average values at all indices for this sector
            df_pe['LS_AVG_TPE'][start_index:end_index] = curr_sector_tpe_avg
            df_pe['LS_AVG_FPE'][start_index:end_index] = curr_sector_fpe_avg


        #New data frame with columns from PE dataframe joined
        df_sp = df_sp.join(df_pe)

        #Drop unwanted fields
        df_sp = df_sp.dropna(axis=0, how='all').drop('Unnamed: 0', axis=1)

        #Rearrange based on ticker symbol
        df_sp = df_sp.sort_values('Symbol')

        #Save for later reference and processing
        df_sp.to_csv(path / 'SP500_SEC_PES.csv', index=False)





# Author: Salyl Bhagwat, Gammath Works
# Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works
# All Rights Reserved
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__author__ = 'Salyl Bhagwat'
__copyright__ = 'Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works'

import yfinance as yf
from pathlib import Path
import pandas as pd
import sys
import time
import os
try:
    from gammath_spot import gammath_get_stocks_summary as gss
    from gammath_spot import gammath_get_stocks_financials as gsf
    from gammath_spot import gammath_get_stocks_history as gsh
    from gammath_spot import gammath_get_stocks_options_data as gso
    from gammath_spot import gammath_get_stcktwts as ggstw
    from gammath_spot import gammath_get_stocks_calendar as gsc
except:
    import gammath_get_stocks_summary as gss
    import gammath_get_stocks_financials as gsf
    import gammath_get_stocks_history as gsh
    import gammath_get_stocks_options_data as gso
    import gammath_get_stcktwts as ggstw
    import gammath_get_stocks_calendar as gsc

class GSD:

    def __init__(self):

        self.Tickers_dir = Path('tickers')

    def get_stocks_data(self, tsymbol):
        #Get data for stock ticker symbol from the internet

        if (len(tsymbol) == 0):
            raise ValueError('Invalid ticker symbol')

        path = self.Tickers_dir / f'{tsymbol}'

        if not path.exists():
            path.mkdir(parents=True, exist_ok=True)

        try:
            #Fetch stocktwits page; Finish getting all data outside of yahoo
            ggstw.get_stocktwits_ticker_info(tsymbol, path)
        except:
            print(f'\nERROR: Failed to get Stocktwits info for {tsymbol}')

        try:
            #Create Yahoo finance ticker handle
            ticker = yf.Ticker(tsymbol)
        except:
            print(f'\nERROR: Failed to create Yahoo ticker handle for {tsymbol}')
            raise RuntimeError('Failed to create Yahoo ticker handle')

        try:
            #Get stock info
            gss.get_ticker_summary(tsymbol, ticker, path)

        except ValueError:
            print(f'Error while getting stock summary for {tsymbol}')

        try:
            #Get stock financials
            gsf.get_ticker_financials(tsymbol, ticker, path)
        except ValueError:
            print(f'Error while getting stock financial data for {tsymbol}')

        try:
            #Get stock options data [NOTE: This is very slow]
            gso.get_options_data(tsymbol, ticker, path)
        except ValueError:
            print(f'Error while getting stock options data for {tsymbol}')
        except RuntimeError:
            print(f'Could not get stock options data for {tsymbol}')

        try:
            #Get calendar info [NOTE: This is very low]
            gsc.get_ticker_calendar(tsymbol, ticker, path)
        except ValueError:
            print(f'Error while getting stock calendar data for {tsymbol}')
        except RuntimeError:
            print(f'Could not get stock calendar data for {tsymbol}')

        try:
            #Get stock history
            result = gsh.get_ticker_history(tsymbol, ticker, path)
        except ValueError:
            print(f'\nError while getting ticker price history for {tsymbol}')
        except RuntimeError:
            print(f'\nCould not get stock price history data for {tsymbol}')

        return



# Author: Salyl Bhagwat, Gammath Works
# Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works
# All Rights Reserved
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__author__ = 'Salyl Bhagwat'
__copyright__ = 'Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works'

import pandas as pd
import numpy as np
import statsmodels.api as sm
from pathlib import Path


def get_ols_signals(tsymbol, df, path):

    MIN_TRADING_DAYS_PER_YEAR = 249

    ols_gscore = 0
    ols_max_score = 0
    ols_signals = ''
    curr_diff_sign = ''
    curr_diff_quantile_str = 'quantile ignored'
    slope_sign_5y = ''
    slope_sign_1y = ''

    #Get the price data for y-axis
    prices_len = len(df.Close)

    #We want at least 5 years of price history for generating 5Y LS line slope
    #OLS line slope is critical to conclude if dollar cost averaging strategy should be used
    if (prices_len < (MIN_TRADING_DAYS_PER_YEAR*5)):
        raise ValueError('price history too short')

    y_vals = np.array(df.Close)
    y_vals_len = len(y_vals)
    try:
        x_vals = sm.add_constant([x for x in range(prices_len)])
    except:
        raise RuntimeError('Stats Models API failure')

    x_vals_len = len(x_vals)

    #OLS using statsmodels API

    #Model last 1 year data
    index_1y = MIN_TRADING_DAYS_PER_YEAR

    try:
        model_1y = sm.OLS(y_vals[(y_vals_len-index_1y):], x_vals[(x_vals_len-index_1y):]).fit()
    except:
        raise RuntimeError('Stats Models API failure')

    #Model last 5 years data
    try:
        model = sm.OLS(y_vals, x_vals).fit()
    except:
        raise RuntimeError('Stats Models API failure')

    #Get yprediction to plot the 1y OLS line along with the price chart
    try:
        y1_predictions = model_1y.predict()
    except:
        raise RuntimeError('Stats Models API failure')

    y1_predictions_len = len(y1_predictions)

    #Get yprediction to plot the 5y OLS line along with price chart
    try:
        y_predictions = model.predict()
    except:
        raise RuntimeError('Stats Models API failure')

    y_predictions_len = len(y_predictions)

    #pd dataframe for 1Y predictions. Will need all elements in same size so need to fill in nan elsewhere
    y1_series = pd.Series(np.nan, pd.RangeIndex(prices_len))

    #Put the 1y predictions in right place
    y1_series[len(y1_series)-y1_predictions_len:] = y1_predictions

    #Get the 1y residual values
    resid_1y = model_1y.resid
    resid_1y_len = len(resid_1y)

    #Get the 5y residual values
    resid_5y = model.resid
    resid_5y_len = len(resid_5y)

    #Get the R2 value for determining goodness-of-fit for 1y OLS
    fit_score_1y = round(model_1y.rsquared, 3)

    #Log the 1y score for debugging

    #Get the R2 value for determining goodness-of-fit for 5y OLS
    fit_score_5y = round(model.rsquared, 3)

    #Log the score for debugging

    #Slope of 1Y OLS line (just need to do y2-y1 to get the direction)
    slope_dir_1y = (y1_predictions[y1_predictions_len-1] - y1_predictions[0])
    if (slope_dir_1y < 0):
        slope_sign_1y = '-ve'
    else:
        slope_sign_1y = '+ve'

    #Slope of OLS line
    slope_dir_5y = (y_predictions[y_predictions_len-1] - y_predictions[0])
    if (slope_dir_5y < 0):
        slope_sign_5y = '-ve'
    else:
        slope_sign_5y = '+ve'

    curr_1y_pdiff = 0
    curr_1y_ndiff = 0
    curr_1y_diff = 0
    residual_1y = 0
    bp_1y = 0
    mp_1y = 0
    tp_1y = 0

    curr_5y_pdiff = 0
    curr_5y_ndiff = 0
    curr_5y_diff = 0
    residual_5y = 0
    bp_5y = 0
    mp_5y = 0
    tp_5y = 0

    ls_1y_ndiff_series = pd.Series(np.nan, pd.RangeIndex(resid_1y_len))
    ls_1y_ndiff_count_index = 0

    ls_1y_pdiff_series = pd.Series(np.nan, pd.RangeIndex(resid_1y_len))
    ls_1y_pdiff_count_index = 0

    ls_5y_ndiff_series = pd.Series(np.nan, pd.RangeIndex(resid_5y_len))
    ls_5y_ndiff_count_index = 0

    ls_5y_pdiff_series = pd.Series(np.nan, pd.RangeIndex(resid_5y_len))
    ls_5y_pdiff_count_index = 0

    #Get info on biggest 1y diff
    for i in range(resid_1y_len):
        residual_1y = resid_1y[i]

        if (residual_1y <= 0):
            curr_1y_ndiff = abs(residual_1y)
            ls_1y_ndiff_series[ls_1y_ndiff_count_index] = curr_1y_ndiff
            ls_1y_ndiff_count_index += 1
        else:
            curr_1y_pdiff = residual_1y
            ls_1y_pdiff_series[ls_1y_pdiff_count_index] = curr_1y_pdiff
            ls_1y_pdiff_count_index += 1

    ls_1y_ndiff_series = ls_1y_ndiff_series.dropna(how='all')
    ls_1y_ndiff_series = ls_1y_ndiff_series.sort_values()

    ls_1y_pdiff_series = ls_1y_pdiff_series.dropna(how='all')
    ls_1y_pdiff_series = ls_1y_pdiff_series.sort_values()


    if (residual_1y <= 0):
        #Get 1y percentile values for -ve residuals
        bp_1y, mp_1y, tp_1y = ls_1y_ndiff_series.quantile([0.25, 0.5, 0.75])
        curr_1y_diff = curr_1y_ndiff
    else:
        #Get 1y percentile values for +ve residuals
        bp_1y, mp_1y, tp_1y = ls_1y_pdiff_series.quantile([0.25, 0.5, 0.75])
        curr_1y_diff = curr_1y_pdiff

    #Get info on biggest 5y diff
    if ((prices_len != y_predictions_len) or (prices_len != len(resid_5y))):
        raise RuntimeError('Price data and prediction data length mismatched')
    else:
        for i in range(prices_len):
            residual_5y = resid_5y[i]

            if (residual_5y <= 0):
                curr_5y_ndiff = abs(residual_5y)
                ls_5y_ndiff_series[ls_5y_ndiff_count_index] = curr_5y_ndiff
                ls_5y_ndiff_count_index += 1
            else:
                curr_5y_pdiff = residual_5y
                ls_5y_pdiff_series[ls_5y_pdiff_count_index] = curr_5y_pdiff
                ls_5y_pdiff_count_index += 1

    ls_5y_ndiff_series = ls_5y_ndiff_series.dropna(how='all')
    ls_5y_ndiff_series = ls_5y_ndiff_series.sort_values()

    ls_5y_pdiff_series = ls_5y_pdiff_series.dropna(how='all')
    ls_5y_pdiff_series = ls_5y_pdiff_series.sort_values()


    if (residual_5y <= 0):
        #Get 5y percentile values for -ve residuals
        bp_5y, mp_5y, tp_5y = ls_5y_ndiff_series.quantile([0.25, 0.5, 0.75])
        curr_5y_diff = curr_5y_ndiff
    else:
        #Get 5y percentile values for +ve residuals
        bp_5y, mp_5y, tp_5y = ls_5y_pdiff_series.quantile([0.25, 0.5, 0.75])
        curr_5y_diff = curr_5y_pdiff


    #Best fit-score is 1; Using 0.9-1.0 as good fit (5y or 1y)
    fits_1y = ((fit_score_1y <= 1) and (fit_score_1y >= 0.9))
    fits_5y = ((fit_score_5y <= 1) and (fit_score_5y >= 0.9))

    #First establish which residual, diff and percentile values we will use based on fitness

    #5y residual is preferred as that is based on larger sample
    if (fits_5y):
        residual = residual_5y
        curr_diff = curr_5y_diff
        bp = bp_5y
        mp = mp_5y
        tp = tp_5y
        fit_score = fit_score_5y
    elif (fits_1y):
        residual = residual_1y
        curr_diff = curr_1y_diff
        bp = bp_1y
        mp = mp_1y
        tp = tp_1y
        fit_score = fit_score_1y
    else:
        residual = residual_5y
        curr_diff = curr_5y_diff
        bp = bp_5y
        mp = mp_5y
        tp = tp_5y
        fit_score = fit_score_5y

    if (slope_dir_1y <= 0):
        #Slope is -ve for last 1 year so no need to check fits or residuals
        ols_gscore -= 10
    else:
        #Finer scoring based on percentile only when there is a fit
        if (fits_5y or fits_1y):
            if (residual <= 0):
                #Below OLS line
                if (curr_diff < mp):
                    curr_diff_quantile_str = 'bottom quantile'

                if (curr_diff >= bp):
                    ols_gscore += 1

                    if (curr_diff >= mp):
                        ols_gscore += 2
                        curr_diff_quantile_str = 'middle quantile'

                    if (curr_diff >= tp):
                        ols_gscore += 2
                        curr_diff_quantile_str = 'top quantile'
            else:
                #Above OLS line
                if (curr_diff < mp):
                    curr_diff_quantile_str = 'bottom quantile'

                if (curr_diff >= bp):
                    ols_gscore -= 1

                    if (curr_diff >= mp):
                        ols_gscore -= 2
                        curr_diff_quantile_str = 'middle quantile'

                        if (curr_diff >= tp):
                            ols_gscore -= 2
                            curr_diff_quantile_str = 'top quantile'

        if (slope_dir_5y <= 0):
            #Slope is -ve for 5y but 1y is +ve so possible sign of recovery
            if (residual <= 0):
                #Below OLS line
                ols_gscore += 3
            else:
                #Above OLS line
                ols_gscore -= 3
        else:
            #Both slopes are +ve
            #Slope is -ve for 5y but 1y is +ve so possible sign of recovery
            if (residual <= 0):
                #Below OLS line
                ols_gscore += 5
            else:
                #Above OLS line
                ols_gscore -= 5


    ols_max_score += 10

    if (residual <= 0):
        curr_diff_sign = '-ve'
    else:
        curr_diff_sign = '+ve'

    ols_grec = f'ols_gscore:{ols_gscore}/{ols_max_score}'

    ols_signals = f'OLS: ols_fit_score:{fit_score},1y_slope:{slope_sign_1y},5y_slope:{slope_sign_5y},curr_diff:{curr_diff_sign},{curr_diff_quantile_str},{ols_grec}'

    #Return OLS lines data in a dataframe for plotting charts
    ols_df = pd.DataFrame({tsymbol: df.Close, 'OLS': y_predictions, 'OLS_1Y': y1_series})

    return ols_df, ols_gscore, ols_max_score, ols_signals



# Author: Salyl Bhagwat, Gammath Works
# Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works
# All Rights Reserved
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__author__ = 'Salyl Bhagwat'
__copyright__ = 'Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works'

import yfinance as yf
from pathlib import Path
import pandas as pd
import sys
import time
import os
try:
    from gammath_spot import gammath_utils as gut
except:
    import gammath_utils as gut

def get_ticker_financials(tsymbol, ticker, path):

    if (len(tsymbol) == 0):
        raise ValueError('Invalid symbol')

    MIN_DELAY_BETWEEN_BATCHES = 1
    MAX_DELAY_BETWEEN_BATCHES = 3

    Tickers_dir = Path('tickers')

    qbs_dont_need_fetch = True
    qcf_dont_need_fetch = True
    qe_dont_need_fetch = True
    qf_dont_need_fetch = True
    reco_dont_need_fetch = True

    if not path.exists():
        path.mkdir(parents=True, exist_ok=True)

    try:
        file_exists = (path / f'{tsymbol}_qbs.csv').exists()

        #Check if file exists and is it from another day
        if file_exists:
            fstat = os.stat(path / f'{tsymbol}_qbs.csv')
            qbs_dont_need_fetch = gut.check_if_same_day(fstat)
        else:
            qbs_dont_need_fetch = False
    except:
        print(f'\nQuarterly balance sheet for ticker {tsymbol} not found')

    try:
        file_exists = (path / f'{tsymbol}_qcf.csv').exists()

        #Check if file exists and is it from another day
        if file_exists:
            fstat = os.stat(path / f'{tsymbol}_qcf.csv')
            qcf_dont_need_fetch = gut.check_if_same_day(fstat)
        else:
            qcf_dont_need_fetch = False
    except:
        qcf_dont_need_fetch = False

    try:
        file_exists = (path / f'{tsymbol}_qe.csv').exists()

        #Check if file exists and is it from another day
        if file_exists:
            fstat = os.stat(path / f'{tsymbol}_qe.csv')
            qe_dont_need_fetch = gut.check_if_same_day(fstat)
        else:
            qe_dont_need_fetch = False
    except:
        qe_dont_need_fetch = False

    try:
        file_exists = (path / f'{tsymbol}_qf.csv').exists()

        #Check if file exists and is it from another day
        if file_exists:
            fstat = os.stat(path / f'{tsymbol}_qf.csv')
            qf_dont_need_fetch = gut.check_if_same_day(fstat)
        else:
            qf_dont_need_fetch = False
    except:
        qf_dont_need_fetch = False

    try:

        file_exists = (path / f'{tsymbol}_reco.csv').exists()

        #Check if file exists and is it from another day
        if file_exists:
            fstat = os.stat(path / f'{tsymbol}_reco.csv')
            reco_dont_need_fetch = gut.check_if_same_day(fstat)
        else:
            reco_dont_need_fetch = False
    except:
        reco_dont_need_fetch = False


    if (qbs_dont_need_fetch and qcf_dont_need_fetch and qe_dont_need_fetch and qf_dont_need_fetch and reco_dont_need_fetch):
        return
    else:

        if not qbs_dont_need_fetch:
            try:
                stock_balance_sheet = ticker.quarterly_balancesheet
                if (len(stock_balance_sheet) > 0):
                    #Save the data for reference and processing
                    stock_balance_sheet.to_csv(path / f'{tsymbol}_qbs.csv')
            except:
                qbs_dont_need_fetch = True

        if not qcf_dont_need_fetch:
            try:
                stock_cash_flow = ticker.quarterly_cashflow
                if (len(stock_cash_flow) > 0):
                    #Save the data for reference and processing
                    stock_cash_flow.to_csv(path / f'{tsymbol}_qcf.csv')
            except:
                qcf_dont_need_fetch = True

        if not qe_dont_need_fetch:
            try:
                stock_earnings = ticker.quarterly_earnings
                #Save the data for reference and processing
                if (len(stock_earnings) > 0):
                    stock_earnings.to_csv(path / f'{tsymbol}_qe.csv')
            except:
                qe_dont_need_fetch = True

        if not qf_dont_need_fetch:
            try:
                stock_financials = ticker.quarterly_financials
                if (len(stock_financials) > 0):
                    #Save the data for reference and processing
                    stock_financials.to_csv(path / f'{tsymbol}_qf.csv')
            except:
                qf_dont_need_fetch = True

        if not reco_dont_need_fetch:
            try:
                stock_recommendations = ticker.recommendations
                if (len(stock_recommendations) > 0):
                    #Save the data for reference and processing
                    stock_recommendations.to_csv(path / f'{tsymbol}_reco.csv')
            except:
                reco_dont_need_fetch = True

    return



# Author: Salyl Bhagwat, Gammath Works
# Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works
# All Rights Reserved
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__author__ = 'Salyl Bhagwat'
__copyright__ = 'Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works'

from pathlib import Path
import pandas as pd


def get_reco_signals(tsymbol, path):

    buy_recos = ('Above Average', 'Accumulate', 'Add', 'Buy', 'Conviction Buy', 'Gradually Accumulate', 'Long-Term Buy', 'Long-term Buy', 'Market Outperform', 'Outperform', 'Outperformer', 'Overweight', 'Positive', 'Sector Outperform', 'Strong Buy', 'Top Pick')

    reco_gscore = 0
    reco_max_score = 0

    try:
        file_exists = (path / f'{tsymbol}_reco.csv').exists()

        #Check if file exists and is it from another day
        if file_exists:
            df = pd.read_csv(path / f'{tsymbol}_reco.csv')
            len_df = len(df)
            if (len_df == 0):
                #This will allow doing our own fundamental analysis.
                reco_max_score = 0
            else:
                #We only want recent data so get shorter of last 25% of recommendations from entire list; 25% to align with approximately quarterly updates
                df_25p_index = len_df - int((len_df * 25) / 100)
                if (df_25p_index < len_df):
                    start_index = df_25p_index
                else:
                    start_index = 0

                #Shorter list for recent recommendations
                shorter_df = df[start_index:len_df]

                total_recos = len(shorter_df)

                buy_count = 0
                sell_count = 0

                #Extract count for buy/sell/positive recommendations
                for grade in shorter_df['To Grade']:
                    if (grade in buy_recos):
                        buy_count += 1
                    else:
                        sell_count += 1


                #Get the percentage of +ve recommendations
                buy_percentage = buy_count*100/total_recos

                #Reduce buy score and increase sell score
                if (buy_percentage < 50):
                    reco_gscore -= 4
                else:
                    reco_gscore += 2

                    if (buy_percentage >= 75):
                        reco_gscore += 2

                reco_max_score += 4

                #Extract count for recent upgrades/downgrades recommendations
                up_count = 0
                down_count = 0

                for action in shorter_df['Action']:
                    if (action == 'up'):
                        up_count += 1
                    elif (action == 'down'):
                        down_count += 1

                #Get the percentage of +ve recommendations
                total_up_down_count = up_count + down_count
                if (total_up_down_count > 0):
                    up_percentage = (up_count*100)/(up_count + down_count)
                else:
                    up_percentage = 0

                #More weightage to recent upgrade/downgrade compared to buy/sell reco
                if (up_percentage < 50):
                    reco_gscore -= 6
                else:
                    reco_gscore += 3

                    if (up_percentage >= 75):
                        reco_gscore += 3

                reco_max_score += 6

        else:
            #This will show 0/10 where expert recommendations don't exist.
            reco_max_score = 0
    except:
        raise RuntimeError('Quarterly Recommendations scoring failed')


    reco_signals = f'reco:reco_gscore:{reco_gscore}/{reco_max_score}'

    return reco_gscore, reco_max_score, reco_signals



# Author: Salyl Bhagwat, Gammath Works
# Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works
# All Rights Reserved
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__author__ = 'Salyl Bhagwat'
__copyright__ = 'Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works'

from pathlib import Path
import pandas as pd
import os

def get_events_info(tsymbol, path):

    event = 'No events found'
    event_dates = []

    #Get the events data from existing file
    if ((path / f'{tsymbol}_calendar.csv').exists()):

        #Get the latest options expiry date
        df = pd.read_csv(path / f'{tsymbol}_calendar.csv', index_col='Unnamed: 0')

        num_rows = len(df.index)
        if (num_rows != 0):
            num_cols = len(df.columns)
            if (num_cols != 0):
                #For now only use Earnings event (happens to be the first event in the calendar)
                event = df.index[0]

                #Extract all event dates
                for i in range(num_cols):
                    event_dates.append(df[df.columns[i]][event])
    else:
        print(f'\nINFO: Events file for {tsymbol} not found')

    events_info = f'{event}: {event_dates}'
    
    return events_info




# Author: Salyl Bhagwat, Gammath Works
# Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works
# All Rights Reserved
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__author__ = 'Salyl Bhagwat'
__copyright__ = 'Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works'

import pandas as pd
from datetime import datetime
from pathlib import Path
from talib import RSI, BBANDS, MACD, MFI, STOCH
import numpy as np

def get_macd_signals(tsymbol, df, path):

    MACD_FAST_PERIOD = 12
    MACD_SLOW_PERIOD = 26
    MACD_SIGNAL_PERIOD = 9

    macd_gscore = 0
    macd_max_score = 0
    curr_count_quantile_str = ''
    curr_diff_quantile_str = ''

    try:
        macd, macd_signal, macd_histogram = MACD(df.Close, MACD_FAST_PERIOD, MACD_SLOW_PERIOD, MACD_SIGNAL_PERIOD)
    except:
        raise RuntimeError('MACD data generation failed')

    macd_len = len(macd)
    if (macd_len <= 0):
        raise ValueError('MACD data length error')

    #Check current MACD trend
    macd_trend = ''
    if (macd_histogram[macd_len-1] > 0):
        macd_trend = 'positive'
    else:
        macd_trend = 'negative'

    #Generally speaking, buy signal is when -ve to +ve crossover is encountered and sell signal is when +ve to -ve crossover is encountered. However, here, we are not using the buy/sell signal; instead we are using the trends and difference to get better price before the crossover is seen i.e. higher dip score during -ve trend and higher premium score during +ve trend

    curr_days_in_positive = 0
    curr_macd_pdiff = 0
    curr_days_in_negative = 0
    curr_macd_ndiff = 0
    max_macd_ndiff = 0

    last_buy_signal_index = 0
    last_sell_signal_index = 0

    macd_neg_days_count_series = pd.Series(np.nan, pd.RangeIndex(macd_len))
    macd_neg_days_count_index = 0

    macd_pos_days_count_series = pd.Series(np.nan, pd.RangeIndex(macd_len))
    macd_pos_days_count_index = 0

    macd_neg_diff_count_series = pd.Series(np.nan, pd.RangeIndex(macd_len))
    macd_neg_diff_count_index = 0

    macd_pos_diff_count_series = pd.Series(np.nan, pd.RangeIndex(macd_len))
    macd_pos_diff_count_index = 0

    #Maintain the count and diff when MACD indicates -ve trend and +ve trend
    for i in range(macd_len-1):
        if ((macd_histogram[i] <= 0) and (macd_histogram[i+1] > 0)):
            #Buy signal
            if (curr_days_in_negative > 0):
                macd_neg_days_count_series[macd_neg_days_count_index] = curr_days_in_negative
                macd_neg_days_count_index += 1

            curr_days_in_positive = 1
            curr_days_in_negative = 0
            last_buy_signal_index = i+1
            curr_macd_pdiff = round(macd_histogram[i+1], 3)
            macd_pos_diff_count_series[macd_pos_diff_count_index] = curr_macd_pdiff
            macd_pos_diff_count_index += 1
        elif ((macd_histogram[i] >= 0) and (macd_histogram[i+1] < 0)):
            #Sell signal

            if (curr_days_in_positive > 0):
                macd_pos_days_count_series[macd_pos_days_count_index] = curr_days_in_positive
                macd_pos_days_count_index += 1

            curr_days_in_negative = 1
            curr_days_in_positive = 0
            last_sell_signal_index = i+1
            curr_macd_ndiff = round(abs(macd_histogram[i+1]), 3)
            macd_neg_diff_count_series[macd_neg_diff_count_index] = curr_macd_ndiff
            macd_neg_diff_count_index += 1
        else:
            if (curr_days_in_positive != 0):
                curr_days_in_positive += 1
                curr_macd_pdiff = round(macd_histogram[i+1], 3)
                macd_pos_diff_count_series[macd_pos_diff_count_index] = curr_macd_pdiff
                macd_pos_diff_count_index += 1
            elif (curr_days_in_negative != 0):
                curr_days_in_negative += 1
                curr_macd_ndiff = round(abs(macd_histogram[i+1]), 3)
                macd_neg_diff_count_series[macd_neg_diff_count_index] = curr_macd_ndiff
                macd_neg_diff_count_index += 1

    #Drop nans and sort the vals
    macd_neg_days_count_series = macd_neg_days_count_series.dropna()
    macd_neg_days_count_series = macd_neg_days_count_series.sort_values()

    macd_neg_diff_count_series = macd_neg_diff_count_series.dropna()
    macd_neg_diff_count_series = macd_neg_diff_count_series.sort_values()

    macd_pos_days_count_series = macd_pos_days_count_series.dropna()
    macd_pos_days_count_series = macd_pos_days_count_series.sort_values()

    macd_pos_diff_count_series = macd_pos_diff_count_series.dropna()
    macd_pos_diff_count_series = macd_pos_diff_count_series.sort_values()

    #Buy/Sell signal moment is only a small part of the equation; No scoring on the start of the indication

    #Check which percentile quarter do current -ve and +ve trend days fall
    if (curr_days_in_negative > 0):
        #Get values for -ve trend days counts at 25/50/75 percentile
        bp, mp, tp = macd_neg_days_count_series.quantile([0.25, 0.5, 0.75])

        #Get values for -ve diff at 25/50/75 percentile
        bp_diff, mp_diff, tp_diff = macd_neg_diff_count_series.quantile([0.25, 0.5, 0.75])

        if (curr_macd_ndiff < mp_diff):
            curr_diff_quantile_str = 'bottom quantile'

        if (curr_macd_ndiff >= mp_diff):
            curr_diff_quantile_str = 'middle quantile'

        if (curr_macd_ndiff >= tp_diff):
            curr_diff_quantile_str = 'top quantile'

        #It has just crossed over to sell side so don't buy at least until we hit 25 percentile of -ve days
        if (curr_days_in_negative < bp):
            macd_gscore -= 10
            curr_count_quantile_str = 'bottom quantile'
        else:
            #Increase buy score at 25, 50 and 75 percentile crossing
            if (curr_days_in_negative >= bp):
                macd_gscore += 1
                curr_count_quantile_str = 'bottom quantile'

            if (curr_days_in_negative >= mp):
                macd_gscore += 2
                curr_count_quantile_str = 'middle quantile'

            if (curr_days_in_negative >= tp):
                macd_gscore += 2
                curr_count_quantile_str = 'top quantile'

            #Check which percentile quarter do current -ve and +ve diff fall
            if (curr_macd_ndiff > 0):

                if (curr_macd_ndiff >= mp_diff):
                    macd_gscore += 2

                if (curr_macd_ndiff >= tp_diff):
                    macd_gscore += 3


    elif (curr_days_in_positive > 0):

        #Get percentile values for +ve trend days counts
        bp, mp, tp = macd_pos_days_count_series.quantile([0.25, 0.5, 0.75])

        #Get percentile values for +ve diff
        bp_diff, mp_diff, tp_diff = macd_pos_diff_count_series.quantile([0.25, 0.5, 0.75])

        if (curr_macd_pdiff <= mp_diff):
            curr_diff_quantile_str = 'bottom quantile'

        if (curr_macd_pdiff > mp_diff):
            curr_diff_quantile_str = 'middle quantile'

        if (curr_macd_pdiff >= tp_diff):
            curr_diff_quantile_str = 'top quantile'

        #It has just crossed over to buy side so buy only until we hit 25 percentile of +ve days
        if (curr_days_in_positive < bp):
            if (curr_macd_pdiff < mp_diff):
                macd_gscore += 5
            else:
                macd_gscore -= 5

            curr_count_quantile_str = 'bottom quantile'

        else:
            #Increase sell score at 25, 50 and 75 percentile crossing
            if (curr_days_in_positive >= bp):
                macd_gscore -= 1
                curr_count_quantile_str = 'bottom quantile'

            if (curr_days_in_positive >= mp):
                macd_gscore -= 2
                curr_count_quantile_str = 'middle quantile'

            if (curr_days_in_positive >= tp):
                macd_gscore -= 2
                curr_count_quantile_str = 'top quantile'

            if (curr_macd_pdiff > 0):

                if (curr_macd_pdiff > mp_diff):
                    macd_gscore -= 2

                if (curr_macd_pdiff >= tp_diff):
                    macd_gscore -= 3


    macd_max_score += 10

    #Get current stock price
    current_price = df['Close'][len(df)-1]

    #Save the last buy and sell signal dates for reference
    macd_buy_signal_date = df['Date'][last_buy_signal_index]
    buy_sig_price = df['Close'][last_buy_signal_index]
    buy_sig_price_str = f'sig_price:%5.3f' % df['Close'][last_buy_signal_index]
    macd_sell_signal_date = df['Date'][last_sell_signal_index]
    sell_sig_price = df['Close'][last_sell_signal_index]
    sell_sig_price_str = f'sig_price:%5.3f' % df['Close'][last_sell_signal_index]

    #If current price is more than price at sell signal then flag it by setting MACD-gScore to -10
    if (macd_trend == 'negative'):
        if (current_price > sell_sig_price):
            macd_gscore = -10

    #Format the strings to log
    macd_grec = f'macd_gscore:{macd_gscore}/{macd_max_score}'

    macd_buy_sell_sig_date = ''
    if (macd_trend == 'positive'):
        macd_buy_sell_stats = f'{buy_sig_price_str},+ve_days_count in {curr_count_quantile_str},curr_diff in {curr_diff_quantile_str}'
    else:
        macd_buy_sell_stats = f'{sell_sig_price_str},-ve_days_count in {curr_count_quantile_str},curr_diff in {curr_diff_quantile_str}'

    macd_signals = f'MACD trend:{macd_trend},{macd_buy_sell_stats},{macd_grec}'

    #Return MACD lines in a dataframe for plotting charts
    macd_df = pd.DataFrame({'MACD': macd, 'MACD_SIGNAL': macd_signal})

    return macd_df, macd_gscore, macd_max_score, macd_signals



# Author: Salyl Bhagwat, Gammath Works
# Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works
# All Rights Reserved
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__author__ = 'Salyl Bhagwat'
__copyright__ = 'Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works'

import pandas as pd
from talib import BBANDS
import numpy as np


def get_bollinger_bands_signals(tsymbol, df, path):

    BBANDS_TIME_PERIOD = 14

    try:
        #Get bollinger bands values
        ub, mb, lb = BBANDS(df.Close, timeperiod=BBANDS_TIME_PERIOD, nbdevup=2, nbdevdn=2, matype=0)
    except:
        raise RuntimeError('Bollinger Band Call Failed')

    bb_len = len(mb)

    bb_gscore = 0
    bb_max_score = 0
    bb_signals = ''

    if (bb_len <= 0):
        bb_max_score += 10
        bb_signals = f'bollinger bands: ERROR'
        raise ValueError('Bollinger Bands call return 0 length data')

    #Get current values for lower, middle and upper bands
    last_val_lb = lb[bb_len-1]
    last_val_mb = mb[bb_len-1]
    last_val_ub = ub[bb_len-1]

    #Get the most recent price from dataframe
    lp = df['Close'][bb_len-1]

    #Check current price level with respect to bollinger bands
    if (lp < last_val_mb):

        bb_avg = 'below average'
        bb_gscore += 4

        if ((last_val_mb - lp) < (abs(lp - last_val_lb))):
            bb_vicinity = 'near middle band'
            bb_gscore += 1
        else:
            #Higher buy weights when near lower band
            bb_vicinity = 'near lower band'
            bb_gscore += 6

    elif (lp > last_val_mb):

        bb_avg = 'above average'
        bb_gscore -= 4

        if ((lp - last_val_mb) < (abs(last_val_ub - lp))):
            bb_vicinity = 'near middle band'
            bb_gscore -= 1
        else:
            #Higher sell weights when near upper band
            bb_vicinity = 'near upper band'
            bb_gscore -= 6
    else:
        bb_avg = 'average'
        bb_vicinity = 'at middle band'

    bb_max_score += 10

    bb_grec = f'bb_gscore:{bb_gscore}/{bb_max_score}'
    bb_signals = f'bollinger bands:{bb_avg},{bb_vicinity},{bb_grec}'

    #Return Bollinger bands in a dataframe for plotting charts
    bb_df = pd.DataFrame({tsymbol: df.Close, 'Upper Band': ub, 'Middle Band': mb, 'Lower Band': lb})

    return bb_df, bb_gscore, bb_max_score, bb_signals



# Author: Salyl Bhagwat, Gammath Works
# Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works
# All Rights Reserved
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__author__ = 'Salyl Bhagwat'
__copyright__ = 'Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works'

import yfinance as yf
from pathlib import Path
import pandas as pd
import time
import os
try:
    from gammath_spot import gammath_utils as gut
except:
    import gammath_utils as gut

def get_ticker_calendar(tsymbol, ticker, path):

    if (len(tsymbol) == 0):
        raise ValueError('Invalid symbol')

    Tickers_dir = Path('tickers')

    calendar_dont_need_fetch = True

    if not path.exists():
        path.mkdir(parents=True, exist_ok=True)

    try:
        file_exists = (path / f'{tsymbol}_calendar.csv').exists()

        #Check if file exists and is it from another day
        if file_exists:
            fstat = os.stat(path / f'{tsymbol}_calendar.csv')
            calendar_dont_need_fetch = gut.check_if_same_day(fstat)
        else:
            calendar_dont_need_fetch = False
    except:
        raise RuntimeError('Error calendar data file')

    if not calendar_dont_need_fetch:
        try:
            stock_calendar_sheet = ticker.calendar
            if (len(stock_calendar_sheet) > 0):
                #Save the data for reference and processing
                stock_calendar_sheet.to_csv(path / f'{tsymbol}_calendar.csv')
        except:
            raise RuntimeError('Error getting calendar data')


    return



# Author: Salyl Bhagwat, Gammath Works
# Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works
# All Rights Reserved
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__author__ = 'Salyl Bhagwat'
__copyright__ = 'Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works'

from pathlib import Path
import pandas as pd

def get_inshp_signals(tsymbol, df_summ):

    inshp_gscore = 0
    inshp_max_score = 0

    try:
        inshp = df_summ['heldPercentInsiders'][0]
    except:
        raise ValueError('heldPercentInsiders not found')

    if (inshp > 0):
        inshp_gscore += 1
    else:
        inshp_gscore -= 1

    inshp_max_score += 1

    #Round it off to take less space displaying the value
    inshp = round(inshp, 3)

    #At some point, we can add percent change. Right now requires to be checked using local old val with new val; REVISIT

    inshp_grec = f'inshp_gscore:{inshp_gscore}/{inshp_max_score}'

    inshp_signals = f'inshp:{inshp},{inshp_grec}'

    return inshp_gscore, inshp_max_score, inshp_signals



# Author: Salyl Bhagwat, Gammath Works
# Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works
# All Rights Reserved
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
__author__= 'Salyl Bhagwat'
__copyright__ = 'Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works'

from setuptools import setup

def readme():
    with open('README.rst') as f:
        return f.read()

setup(name='gammath_spot',
      version='2.2',
      description='Stock Price-Opining Tools',
      long_description=readme(),
      long_description_content_type='text/markdown',
      classifiers=['Development Status :: 4 - Beta', 'License :: OSI Approved :: GNU General Public License v3 (GPLv3)', 'Programming Language :: Python :: 3.8', 'Topic :: Office/Business :: Financial :: Investment'],
      url='https://github.com/salylgw/gammath_spot.git',
      author='Salyl Bhagwat',
      author_email='salylgw@gmail.com',
      packages=['gammath_spot'],
      install_requires=['numpy', 'pandas', 'ta-lib', 'yfinance', 'pykalman', 'statsmodels', 'matplotlib',],
      include_package_data=True,
      entry_points = {
          'console_scripts': ['gammath_scraper=gammath_spot.gammath_stocks_data_scraper:main','gammath_scorer=gammath_spot.gammath_stocks_analyzer_and_scorer:main'],
      },
      zip_safe=False)




# Author: Salyl Bhagwat, Gammath Works
# Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works
# All Rights Reserved
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__author__ = 'Salyl Bhagwat'
__copyright__ = 'Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works'

import pandas as pd
from talib import STOCH

def get_stochastics_slow_signals(tsymbol, df):

    STOCH_FAST_PERIOD = 14
    STOCH_SLOW_PERIOD = 3
    STOCH_OVERSOLD_LEVEL = 20
    STOCH_OVERBOUGHT_LEVEL = 80

    stoch_gscore = 0
    stoch_max_score = 0
    stoch_slow_signals = ''
    stoch_lvl = ''


    try:
        slowk, slowd = STOCH(df.High, df.Low, df.Close, fastk_period=STOCH_FAST_PERIOD, slowk_period=STOCH_SLOW_PERIOD, slowk_matype=0, slowd_period=STOCH_SLOW_PERIOD, slowd_matype=0)
    except:
        raise RuntimeError('Stochastics data generation failed')

    stoch_len = len(slowd)

    if (stoch_len <= 0):
        raise ValueError('Stochastics data length error')

    stoch_d_curr_val = slowd[stoch_len-1]
    slowd_ds = slowd.describe()
    slowd_mean = slowd_ds['mean']

    if (stoch_d_curr_val < slowd_mean):
        stoch_lvl = 'below average'
        stoch_gscore += 2
    elif (stoch_d_curr_val > slowd_mean):
        stoch_lvl = 'above average'
        stoch_gscore -= 2
    else:
        stoch_lvl = 'average'

    stoch_max_score += 2

    #Check for stochastic oversold/overbought levels
    if (stoch_d_curr_val <= STOCH_OVERSOLD_LEVEL):
        stoch_lvl = 'oversold'
        stoch_gscore += 3
    elif (stoch_d_curr_val >= STOCH_OVERBOUGHT_LEVEL):
        stoch_lvl = 'overbought'
        stoch_gscore -= 3

    stoch_max_score += 3

    last_crossover_index = 0

    for i in range(stoch_len-1):
        #Detect crossovers
        if (((slowk[i] < slowd[i]) and (slowk[i+1] > slowd[i+1])) or ((slowk[i] > slowd[i]) and (slowk[i+1] < slowd[i+1]))):
            last_crossover_index = i+1

    #Get crossover date
    stoch_crossover_date = df['Date'][last_crossover_index]

    #Keep slowd value for reference
    stoch_slowd = round(slowd[stoch_len-1], 3)

    stoch_grec = f'stoch_gscore:{stoch_gscore}/{stoch_max_score}'

    stoch_slow_signals = f'stochs: {stoch_lvl},slowd:{stoch_slowd},l_xovr_date:{stoch_crossover_date},{stoch_grec}'

    #Return slowk and slowd in a dataframe for plotting charts
    stoch_df = pd.DataFrame({'SLOWK': slowk, 'SLOWD': slowd})

    return stoch_df, stoch_gscore, stoch_max_score, stoch_slow_signals



# Author: Salyl Bhagwat, Gammath Works
# Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works
# All Rights Reserved
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__author__ = 'Salyl Bhagwat'
__copyright__ = 'Copyright (c) 2021-2022, Salyl Bhagwat, Gammath Works'

import pandas as pd
import numpy as np

def get_peg_signals(tsymbol, df_summ):

    peg_gscore = 0
    peg_max_score = 0

    try:
        peg = round(df_summ['pegRatio'][0], 3)
        if (np.isnan(peg)):
            peg_string = 'No PEG data'
        else:
            peg_string = f'{peg}'
            if (peg > 0):
                if (peg < 3):
                    peg_gscore += 1
            else:
                peg_gscore -= 1
    except:
        peg_string = 'No PEG data'


    peg_max_score += 1

    peg_grec = f'peg_gscore:{peg_gscore}/{peg_max_score}'

    peg_signals = f'PEG:{peg_string},{peg_grec}'

    return peg_gscore, peg_max_score, peg_signals
